<?php
/**
 * @file    class-getlocations-lite-admin.php
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 */

class Getlocations_Lite_Admin {

  /**
   * The ID of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param string  $plugin_name The name of this plugin.
   * @param string  $version     The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {

    $this->plugin_name = $plugin_name;
    $this->version = $version;

    $this->load_dependencies();

    /* add settings to plugin page */
    add_filter('plugin_action_links_' . plugin_basename(GETLOCATIONS_LITE_PLUGIN_FILE), array($this, 'add_action_links'));

    $this->map_plugins = get_option('getlocations_lite_plugins');
    if ( isset($this->map_plugins['leaflet_plugin_kml']) && $this->map_plugins['leaflet_plugin_kml'] ) {
      add_filter('upload_mimes', array($this, 'add_kmlmime_types'), 1, 1);
    }

  }

  /**
   * Load the required dependencies for the Admin facing functionality.
   *
   * Include the following files that make up the plugin:
   *
   * - Getlocations_Lite_Admin_Settings . Registers the admin settings and page.
   *
   *
   * @since    1.0.0
   * @access   private
   */
  private function load_dependencies() {

    /**
     * The class responsible for orchestrating the actions and filters of the
     * core plugin.
     */
    require_once plugin_dir_path( dirname( __FILE__ ) ) .  'admin/class-getlocations-lite-settings.php';

  }

  /**
   * Register the stylesheets for the admin area.
   *
   * @since    1.0.0
   */
  public function enqueue_styles() {

    /**
     *
     * An instance of this class should be passed to the run() function
     * defined in Getlocations_Lite_Loader as all of the hooks are defined
     * in that particular class.
     *
     * The Getlocations_Lite_Loader will then create the relationship
     * between the defined hooks and the functions defined in this
     * class.
     */

    wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet-css', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/leaflet/leaflet.min.css', array(), '1.6.0', 'all' );

    if ( isset($this->map_plugins['leaflet_plugin_fullscreen']) && $this->map_plugins['leaflet_plugin_fullscreen'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_fullscreen', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.fullscreen/Control.FullScreen.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_mouseposition']) && $this->map_plugins['leaflet_plugin_mouseposition'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_mouseposition', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MousePosition/L.Control.MousePosition.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_pancontrol']) && $this->map_plugins['leaflet_plugin_pancontrol'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_pancontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Pancontrol/L.Control.Pan.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_zoomslider']) && $this->map_plugins['leaflet_plugin_zoomslider'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_zoomslider', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.zoomslider/L.Control.Zoomslider.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_magnifyingglass']) && $this->map_plugins['leaflet_plugin_magnifyingglass'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_magnifyingglass', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MagnifyingGlass/css/leaflet.magnifyingglass.min.css', array(), $this->version, 'all' );
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_magnifyingglasscontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MagnifyingGlass/css/Control.MagnifyingGlass.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_minimap']) && $this->map_plugins['leaflet_plugin_minimap'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_minimap', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.layerscontrol-minimap/control.layers.minimap.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_graticule']) && $this->map_plugins['leaflet_plugin_graticule'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_graticule', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.latlng-graticule/graticule.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_markercluster']) && $this->map_plugins['leaflet_plugin_markercluster'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_markercluster', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.markercluster/MarkerCluster.min.css', array(), $this->version, 'all' );
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_markercluster_default', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.markercluster/MarkerCluster.Default.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_terminator']) && $this->map_plugins['leaflet_plugin_terminator'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_terminator', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Terminator/terminator.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_awesome']) && $this->map_plugins['leaflet_plugin_awesome'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_awesome_markers', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.awesome-markers/leaflet.awesome-markers.min.css', array(), $this->version, 'all' );
      wp_enqueue_style( GETLOCATIONS_COMMON . '-ion', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/'  . 'css/ionicons.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_geocoder']) && $this->map_plugins['leaflet_plugin_geocoder'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_geocoder', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet-control-geocoder/Control.Geocoder.min.css ', array(), '1.13.0', 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_kml']) && $this->map_plugins['leaflet_plugin_kml'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_kml', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.kml/kml.min.css', array(), $this->version, 'all' );
    }
    wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/getlocations-lite-admin.min.css', array(), $this->version, 'all' );
  }


  /**
   * Register the JavaScript for the admin area.
   *
   * @since    1.0.0
   */
  public function enqueue_scripts() {

    /**
     * An instance of this class should be passed to the run() function
     * defined in Getlocations_Lite_Loader as all of the hooks are defined
     * in that particular class.
     *
     * The Getlocations_Lite_Loader will then create the relationship
     * between the defined hooks and the functions defined in this
     * class.
     */

    wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet-js', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/leaflet/leaflet.js' , array(), '1.6.0', FALSE );
    wp_enqueue_script( GETLOCATIONS_COMMON . '-providers-js', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/leaflet-providers/leaflet-providers.js', array(), '1.10.1', FALSE );

    // Plugins
    if ( isset($this->map_plugins['leaflet_plugin_fullscreen']) && $this->map_plugins['leaflet_plugin_fullscreen'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_fullscreen', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.fullscreen/Control.FullScreen.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_mouseposition']) && $this->map_plugins['leaflet_plugin_mouseposition'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_mouseposition', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MousePosition/L.Control.MousePosition.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_pancontrol']) && $this->map_plugins['leaflet_plugin_pancontrol'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_pancontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Pancontrol/L.Control.Pan.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_zoomslider']) && $this->map_plugins['leaflet_plugin_zoomslider'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_zoomslider', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.zoomslider/L.Control.Zoomslider.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_magnifyingglass']) && $this->map_plugins['leaflet_plugin_magnifyingglass'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_magnifyingglass', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MagnifyingGlass/js/leaflet.magnifyingglass.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_magnifyingglasscontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MagnifyingGlass/js/Control.MagnifyingGlass.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_minimap']) && $this->map_plugins['leaflet_plugin_minimap'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_minimap', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.layerscontrol-minimap/L.Control.Layers.Minimap.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_graticule']) && $this->map_plugins['leaflet_plugin_graticule'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_graticule', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.latlng-graticule/leaflet.latlng-graticule.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_graticulecontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.latlng-graticule/Control.LatLngGraticule.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_markercluster']) && $this->map_plugins['leaflet_plugin_markercluster'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_markercluster', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.markercluster/leaflet.markercluster.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_terminator']) && $this->map_plugins['leaflet_plugin_terminator'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_terminator', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Terminator/L.Terminator.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_terminatorcontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Terminator/Control.Terminator.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_awesome']) && $this->map_plugins['leaflet_plugin_awesome'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_awesome_markers', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.awesome-markers/leaflet.awesome-markers.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_geocoder']) && $this->map_plugins['leaflet_plugin_geocoder'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_geocoder', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet-control-geocoder/Control.Geocoder.min.js', array(), '1.13.0', FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_kml']) && $this->map_plugins['leaflet_plugin_kml'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_kml', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.kml/L.KML.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_kmlcontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.kml/Control.Kml.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-media-modal-kml', plugin_dir_url( __FILE__ ) . 'js/getlocations-lite-media-modal-kml.min.js', array( 'jquery', 'media-views' ), $this->version, FALSE );
      wp_enqueue_media();
    }
    wp_enqueue_script( $this->plugin_name . '-geo', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/getlocations-lite-geo.min.js', array( 'jquery' ), $this->version, FALSE );
    wp_enqueue_script( $this->plugin_name . '-admin', plugin_dir_url( __FILE__ ) . 'js/getlocations-lite-admin.min.js', array( 'jquery' ), $this->version, TRUE );
    wp_enqueue_script( $this->plugin_name . '-jqrun', plugin_dir_url( __FILE__ ) . 'js/getlocations-lite-jqrun.min.js', array( 'jquery' ), $this->version, TRUE );
  }

  /**
   * Add Settings link to plugins page
   *
   * @param [array] $links
   * @return array
   */
  public function add_action_links( $links ) {
    $mylinks = array('<a href="' . admin_url( 'admin.php?page=getlocations_lite_settings' ) . '">' . __('Settings', 'getlocations-lite') . '</a>');
     return array_merge( $links, $mylinks );
  }

  public function add_kmlmime_types($mime_types) {
    // $mime_types['kml'] = 'application/vnd.google-earth.kml+xml';
    // $mime_types['kmz'] = 'application/vnd.google-earth.kmz';
    $mime_types['kml'] = 'text/xml';
    $mime_types['kmz'] = 'application/zip';
    return $mime_types;
  }

}
