<?php
/**
 * The settings of the plugin.
 * @file    class-getlocations-lite-settings.php
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 */

class Getlocations_Lite_Admin_Settings {

  /**
   * The ID of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param      string    $plugin_name       The name of this plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct( $plugin_name, $version ) {

    define('GETLOCATIONS_LITE_ADMIN_URL', $_SERVER["REQUEST_SCHEME"] . '://' . $_SERVER["HTTP_HOST"] . $_SERVER["SCRIPT_NAME"] . '?page=getlocations_lite_settings');


    $this->plugin_name = $plugin_name;
    $this->version = $version;

    // bring in some data
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/common_vars.php');
    $this->default_providers = $default_providers;
    $this->default_settings = $default_settings;
    $this->default_settings_mapped = $default_settings_mapped;
    $this->bool = $bool;
    $this->plugins_bool = $plugins_bool;
    $this->allowed_fields = $allowed_fields;
    $this->checkbox_fields = $checkbox_fields;
    $this->icon_types = $icon_types;
    $this->icon_effects = $awesome_effects;
    #$this->icon_colors = $awesome_icon_colors;
    $this->icon_colors = $htmlcolors;
    $this->icon_sizes = $awesome_sizes;
    $this->marker_colors = $awesome_marker_colors;
    $this->marker_types = $marker_types;
    $this->default_leaflet_plugin_settings = $default_leaflet_plugin_settings;
    $this->default_leaflet_plugin_data = $default_leaflet_plugin_data;
    $this->default_leaflet_plugin_keys = $default_leaflet_plugin_keys;
    $this->scaleunits = $scaleunits;
    $this->zoomlevels = $zoomlevels;
    $this->controlpositions = $controlpositions;
    $this->controlshowstate = $controlshowstate;
    $this->default_geocoder_services = $default_geocoder_services;
    $this->getlocations_lite_geocoders = get_option('getlocations_lite_geocoders');
    $this->default_marker = $default_marker;

    // bring in some data
    $mp  = get_option('getlocations_lite_providers', $default_providers);
    $map_key_data = get_option('getlocations_lite_map_keys');
    // ensure that the api key is brought in to map providers
    if ( isset($map_key_data['Thunderforest']) ) {
      $thunderforest_key = $map_key_data['Thunderforest'];
    }
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/providers.php');
    foreach ( $mp AS $k => $v  ) {
      if ( isset($default_leaflet_providers_data[$k]) && $v == '1' ) {
        $mp[$k] = $default_leaflet_providers_data[$k];
      }
    }
    $this->map_providers = $mp;
    $this->map_plugins = get_option('getlocations_lite_plugins');


  }

  /**
   * This function adds a menu.
   */
  public function setup_plugin_options_menu() {
    add_options_page(
      __('Getlocations Lite Settings', 'getlocations-lite'), // The title to be displayed in the browser window for this page.
      __('Getlocations Lite', 'getlocations-lite'),          // The text to be displayed for this menu item
      'manage_options',                                      // Which type of users can see this menu item
      'getlocations_lite_settings',                          // The unique ID - that is, the slug - for this menu item
      array( $this, 'render_settings_page_content')          // The name of the function to call when rendering this menu's page
    );
  }

  /**
   * Renders a simple page to display for the theme menu defined above.
   */
  public function render_settings_page_content( $temp_tab = '' ) {
    // bring in some data

    include_once(GETLOCATIONS_LITE_PLUGIN_DIR . 'includes/class-getlocations-lite-share.php');
    $share = new Getlocations_Lite_Share();

    $allowed_tabs = array('default_settings','providers','plugins','markers','help');
    $allowed_formactions = array('markers_save');
    if ( isset( $_REQUEST['formaction'] ) && in_array( $_REQUEST['formaction'], $allowed_formactions, TRUE ) ) {
      $action = $_REQUEST['formaction'];
      if ( $action == 'markers_save' ) {
        // nonce
        if (! wp_verify_nonce($_REQUEST['_wpnonce'], $action)) {
          // uhoh
          add_settings_error('getlocations_lite_badnonce', 'settings', __('Failed', 'getlocations-lite'), 'error');
        }
        else {
          $allmarkers = array_keys($share->get_all_marker_data());
          if ( isset( $_REQUEST['default_marker'] ) && in_array($_REQUEST['default_marker'], $allmarkers, TRUE ) ) {
            $default_marker = sanitize_text_field($_REQUEST['default_marker']);
          }
          else {
            $default_marker = $this->default_marker;
          }

          if (! empty($default_marker)) {
            delete_option('getlocations_lite_default_marker');
            add_option('getlocations_lite_default_marker', $default_marker);
            add_settings_error('getlocations_lite_markers_save', 'settings_updated', __('Markers settings saved', 'getlocations-lite'), 'updated');
          }
          $reset = FALSE;
          if ( isset($_REQUEST['reset']) && $_REQUEST['reset'] ) {
            $reset = TRUE;
          }
          $temp_tab = 'markers';
        }
      }
    } // formaction

    // Tabs
    if( isset( $_REQUEST['tab'] ) && in_array( $_REQUEST['tab'], $allowed_tabs, TRUE ) ) {
      $active_tab = $_REQUEST[ 'tab' ];
    }
    elseif (! empty( $temp_tab ) && in_array( $temp_tab, $allowed_tabs, TRUE ) ) {
      $active_tab = $temp_tab;
    }
    else {
      $active_tab = "default_settings";
    }

    // right, got tab
    $template_path = $this->process_template('tabs.php');
    include($template_path);

    if ( $active_tab == 'providers' ) {
      $template_path = $this->process_template('providers_form.php');
      include($template_path);
    }
    elseif  ( $active_tab == 'markers' ) {
      $title = __('Map Markers', 'getlocations-lite');
      $formaction = "markers_save";
      $getlocations_lite_default_marker = get_option('getlocations_lite_default_marker');
      $markerdir = GETLOCATIONS_LITE_PLUGIN_DIR . 'public' . DIRECTORY_SEPARATOR . 'markers';
      $markerurl = GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/markers';
      $template_path = $this->process_template('markers.php');
      include($template_path);
    }
    elseif ( $active_tab == 'plugins' ) {
      $template_path = $this->process_template('plugins_form.php');
      include($template_path);
    }
    elseif ( $active_tab == 'help' ) {
      $template_path = $this->process_template('help.php');
      include($template_path);
    }
    else {
      $map_plugins_opts = $share->build_map_plugins_opts();
      $template_path = $this->process_template('default_settings_form.php');
      include($template_path);
    }

    echo '</div>';
  }

  /**
   * Defines the content of the  Default Settings Tab
   *
   * @return void
   */
  public function initialize_default_settings() {
    include_once(GETLOCATIONS_LITE_PLUGIN_DIR . 'includes/class-getlocations-lite-share.php');
    $share = new Getlocations_Lite_Share();

    if( FALSE == get_option( 'getlocations_lite_default_settings' ) ) {
      add_option( 'getlocations_lite_default_settings', $this->default_settings );
    }

    $whichpage = 'getlocations_lite_default_settings';
    $saveto = 'getlocations_lite_default_settings';

    // leaflet_preview_section
    // map should show enabled furniture and width/height and used to set default zoom and center
    $maps = array();
    $def = get_option('getlocations_lite_providers');
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/providers.php');
    foreach ( $default_leaflet_providers_data AS $k => $v ) {
      if ( isset( $def[ $k ] ) && $def[ $k ] > 0 ) {
        $maps[$k] = $v['label'];
      }
    }
    // leaflet_default_settings_section
    $whichsection = 'leaflet_default_settings_section';
    $whichtitle = __( 'Default Leaflet map Settings', 'getlocations-lite' );
    $desc_callback = 'default_settings_callback';
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/leaflet_default_settings_section.php');
    // Finally, we register the fields with WordPress

    // leaflet_plugin_settings_section
    $whichsection = 'leaflet_plugins_settings_section';
    $whichpage = 'getlocations_lite_default_settings';
    $saveto = 'getlocations_lite_plugins_settings';
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/leaflet_plugins_settings_section.php');

    // Finally, we register the fields with WordPress
    register_setting(
      'getlocations_lite_default_settings',
      'getlocations_lite_default_settings',
      array('sanitize_callback' =>
        array($this, 'validate_getlocations_lite_default_settings')
      )
    );
    register_setting(
      'getlocations_lite_default_settings',
      'getlocations_lite_plugins_settings',
      array('sanitize_callback' =>
        array($this, 'validate_getlocations_lite_plugins_settings')
      )
    );

  }

  public function initialize_providers() {
    // bring in some data
    if( FALSE == get_option( 'getlocations_lite_providers' ) ) {
      add_option( 'getlocations_lite_providers', $this->default_providers );
    }
    if (FALSE == get_option('getlocations_lite_map_keys')) {
      add_option('getlocations_lite_map_keys', array() );
    }
    if (FALSE == get_option('getlocations_lite_geocoders')) {
      add_option('getlocations_lite_geocoders', array() );
    }

    // leaflet_map_keys_section
    $whichpage = 'getlocations_lite_providers';
    $saveto = 'getlocations_lite_map_keys';
    $whichsection = 'leaflet_map_keys_section';
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/leaflet_map_keys_section.php');

    // leaflet_providers_section
    $whichpage = 'getlocations_lite_providers';
    $saveto = 'getlocations_lite_providers';
    $whichsection = 'leaflet_providers_section';
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/leaflet_providers_section.php');

    // geocoders are also on this page
    // leaflet_geocoders_section
    $whichpage = 'getlocations_lite_providers';
    $saveto = 'getlocations_lite_geocoders';
    $whichsection = 'leaflet_geocoders_section';
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/leaflet_geocoders_section.php');

    // Finally, we register the fields with WordPress. group is getlocations_lite_providers
    register_setting(
      'getlocations_lite_providers',
      'getlocations_lite_map_keys',
      array('sanitize_callback' =>
        array($this, 'validate_getlocations_lite_map_keys')
      )
    );

    register_setting(
      'getlocations_lite_providers',
      'getlocations_lite_geocoders',
      array('sanitize_callback' =>
        array($this, 'validate_getlocations_lite_geocoders')
      )
    );

    register_setting(
      'getlocations_lite_providers',
      'getlocations_lite_providers',
      array('sanitize_callback' =>
        array($this, 'validate_getlocations_lite_providers')
      )
    );

  }

  public function initialize_leaflet_plugins() {
    // bring in some data
    if( FALSE == get_option( 'getlocations_lite_plugins' ) ) {
      add_option( 'getlocations_lite_plugins', array() );
    }
    $whichpage = 'getlocations_lite_plugins';
    $saveto = 'getlocations_lite_plugins';
    $whichsection = 'leaflet_plugins_section';
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/leaflet_plugins_section.php');

    register_setting(
      'getlocations_lite_plugins',
      'getlocations_lite_plugins',
      array('sanitize_callback' =>
        array($this, 'validate_getlocations_lite_plugins')
      )
    );
  }

  public function leaflet_map_keys_callback() {
    echo '<p>' . __( 'Map Keys.', 'getlocations-lite' ) . '</p>';
  }

  public function default_settings_callback() {
    echo '<p>' . __( 'Default settings for Getlocations maps.', 'getlocations-lite' ) . '</p>';
  }

  public function create_map_callback() {
    echo '<p>' . __( 'Settings for Getlocations maps.', 'getlocations-lite' ) . '</p>';
  }

  public function leaflet_providers_callback() {
    echo '<p>' . __( 'You can enable or disable Leaflet Map Providers.', 'getlocations-lite' ) . '</p>';
  }
  public function leaflet_plugins_callback() {
    echo '<p>' . __( 'You can enable or disable Leaflet Plugins.', 'getlocations-lite' ) . '</p>';
  }

  public function leaflet_plugin_settings_callback() {
    echo '<p>' . __( 'Per map settings for Leaflet plugins. Please visit the Leaflet Plugins Tab to enable/disable.', 'getlocations-lite' ) . '</p>';
  }

  public function leaflet_geocoders_callback() {
    $plugins = get_option('getlocations_lite_plugins');
    if ( isset($plugins['leaflet_plugin_geocoder']) && $plugins['leaflet_plugin_geocoder'] ) {
      $desc = '<p>' . __( 'Available Geocoder Services.', 'getlocations-lite' ) . '</p>';
    }
    else {
      $desc = '<p>' . __( 'Leaflet Geocoder Plugin needs to be enabled onthe Plugins Tab.', 'getlocations-lite' ) . '</p>';
    }
    echo $desc;
  }

  /**
   * generates a form textbox
   *
   * @param [array] $args must have opt and id keys, desc is optional
   * @return void
   */
  public function text_element_callback( $args ) {
    $options = get_option( $args['opt'] );
    if (! isset($options[$args['id']])) {
      $options[$args['id']] = ( isset($args['default_opt']) ? $args['default_opt'] : '' );
    }
    $name = $args['opt'].'['.$args['id'].']';
    $widthclass = "";
    if ( isset($args['widthclass']) ) {
      $widthclass = ' class="' . $args['widthclass'] . '"';
    }
    // Render the output
    $html = '<input type="text" id="'.$args['id'].'" name="'.$name.'" value="'.$options[$args['id']].'"'.$widthclass.' />';
    if ( isset($args['desc']) ) {
      $html .= '<br/><label for="'.$args['id'].'">'.$args['desc'].'</label>';
    }
    echo $html;
  }

  /**
   * generates a form textarea
   *
   * @param [array] $args must have opt and id keys, desc is optional
   * @return void
   */
  public function textarea_element_callback( $args ) {
    $options = get_option( $args['opt'] );
    if (! isset($options[$args['id']])) {
      $options[$args['id']] = (isset($args['default_opt']) ? $args['default_opt'] : '');
    }
    $name = $args['opt'].'['.$args['id'].']';
    // Render the output
    if ( ! isset($args['rows']) ) {
      $args['rows'] = 5;
    }
    if ( ! isset($args['cols']) ) {
      $args['cols'] = 30;
    }

    $html = '<textarea cols="'.$args['cols'].'" rows="'.$args['rows'].'" id="'.$args['id'].'" name="'.$name.'">';
    $html .= $options[$args['id']];
    $html .= '</textarea>';

    if ( isset($args['desc']) ) {
      $html .= '<br/><label for="'.$args['id'].'">'.$args['desc'].'</label>';
    }
    echo $html;
  }

  /**
   * generates a form checkbox
   *
   * @param [array] $args must have opt and id keys, desc is optional
   * @return void
   */
  public function checkbox_element_callback($args) {
    $options = get_option( $args['opt'] );
    if (! isset($options[$args['id']])) {
      $options[$args['id']] = (isset($args['default_opt']) ? $args['default_opt'] : '0');
    }
    // Render the output
    $name = $args['opt'].'['.$args['id'].']';
    $html = '<input type="checkbox" id="'.$args['id'].'" name="'.$name.'" value="1"' . checked( '1', $options[$args['id']], '0' ) . '/>';
    if ( isset($args['desc']) ) {
      $html .= '&nbsp;<label for="'.$args['id'].'">'.$args['desc'].'</label>';
    }
    echo $html;
  }

  /**
   * Generates a dropdown
   *
   * @param [array] $args must have opt, select_args and id keys, desc is optional
   * @return void
   */
  public function select_element_callback( $args ) {
    $options = get_option( $args['opt'] );
    if (! isset($options[$args['id']])) {
      $options[$args['id']] = (isset($args['default_opt']) ? $args['default_opt'] : '');
    }
    $name = $args['opt'].'['.$args['id'].']';
    $select_opts = $args['select_opts'];
    $html = '<select id="'.$args['id'].'" name="'.$name.'">';
    foreach ( $select_opts AS $k => $v ) {
      $html .= '<option value="'.$k.'"'.selected( $options[$args['id']], $k, FALSE).'>'.$v.'</option>';
    }
    $html .= '</select>';
    if ( isset($args['desc']) ) {
      $html .= '<br/><label for="'.$args['id'].'">'.$args['desc'].'</label>';
    }
    echo $html;
  }

  /**
   * Place a map on the default_settings page
   *
   *
   */
  public function map_element_callback( $args ) {
    include_once(GETLOCATIONS_LITE_PLUGIN_DIR . 'includes/class-getlocations-lite-share.php');
    $share = new Getlocations_Lite_Share();

    static $getlocations_lite_map_id;
    if ( empty( $getlocations_lite_map_id ) ) {
      $getlocations_lite_map_id =1;
    }
    else {
      $getlocations_lite_map_id++;
    }
    $map_opts = $share->build_map_opts();

    // do not need these in js
    unset($map_opts["leaflet_library"]);
    unset($map_opts["leaflet_url"]);
    unset($map_opts["leaflet_css"]);
    unset($map_opts["leaflet_providers_library"]);
    unset($map_opts["leaflet_providers_url"]);

    $map_layers = array();
    // the layers_ entries in $map_opts are for the layerscontrol and so need to have entries in $map_layers which provides the map details from $this->map_providers
    foreach ( $map_opts AS $k => $v ) {
      if ( preg_match("/^layers_/", $k) && $v ) {
        $l = preg_replace("/^layers_/", '', $k);
        $l = preg_replace("/_/", '.', $l);
        $map_layers[$l] = $this->map_providers[$l];
      }
    }
    // if there is nothing there set the default
    if ( count($map_layers) < 1 ) {
      $l = $this->default_settings['map_provider'];
      $map_layers[$l] = $this->map_providers[$l];
    }
    // get the getlocations_pro_providers and top up $map_layers
    $getlocations_lite_providers = get_option('getlocations_lite_providers', $this->default_settings['map_provider']);
    foreach ( $getlocations_lite_providers AS $k => $v ) {
      if (! isset( $map_layers[$k] ) && $v ) {
        $map_layers[$k] = $this->map_providers[$k];
      }
    }

    $map_plugins_opts = $share->build_map_plugins_opts();
    // do camelcase conversion in $map_opts2 and bool TRUE/FALSE
    $map_opts2 = $share->get_map_opts_mapped($map_opts);

    // prepare and generate json
    $map_opts_arr = array(
      'map_opts' => $map_opts2,
      'map_layers' => $map_layers,
      'map_plugins_opts' => $map_plugins_opts
    );
    $map_opts_js = json_encode($map_opts_arr);


    $inline_data = 'var getlocations_lite_settings = [];' . "\n";
    $inline_data .= 'getlocations_lite_settings["key_'.$getlocations_lite_map_id.'"] = '.$map_opts_js.';'."\n";
    $inline_data .= 'var getlocations_lite_iconinfo = [];' . "\n";
    $this->admin_enqueue_inline($inline_data);
    add_action('admin_enqueue_scripts', 'admin_enqueue_inline');

    $template_path = $this->process_template('map.php');
    include($template_path);
  }

  /**
   * generates a form checkbox
   *
   * @param [array] $args must have opt and id keys, desc is optional
   * @return void
   */
  public function hidden_element_callback($args) {
    $options = get_option( $args['opt'] );
    if (! isset($options[$args['id']])) {
      $options[$args['id']] = (isset($args['default_opt']) ? $args['default_opt'] : '');
    }
    // Render the output
    $name = $args['opt'].'['.$args['id'].']';
    $html = '<input type="hidden" id="'.$args['id'].'" name="'.$name.'" value="' . $options[$args['id']] . '" />';
    if ( isset($args['desc']) ) {
      $html .= '&nbsp;<label for="'.$args['id'].'">'.$args['desc'].'</label>';
    }
    echo $html;
  }

  /**
   * generates a media button with hidden fields, opens a modal
   *
   * @param [array] $args must have opt and id keys, desc is optional
   * @return void
   */
  public function kml_media_selector_callback($args) {
    $options = get_option($args['opt']);
    if (! isset($options[$args['id']])) {
      $options[$args['id']] = (isset($args['default_opt']) ? $args['default_opt'] : '');
    }
    if (! isset($options['kml_filename'])) {
      $options['kml_filename'] = '';
    }
    #wp_enqueue_media();
    add_filter('restrict_mimes', array($this, 'restrict_kmlmime_types'), 1, 1);
    $name = $args['opt'].'['.$args['id'].']';
    // kml url
    $html = '<input type="hidden" id="'.$args['id'].'" name="'.$name.'" value="' . $options[$args['id']] . '" />';
    $html .= '<input type="hidden" id="kml_filename" name="' . $args['opt'] . '['. 'kml_filename' . ']' . '" value="'.$options['kml_filename'].'" />';
    $html .= '<input type="button" class="button" id="open-kml-media-lib" value="'. __('Select KML File').'" />';
    $html .= '<span class="kml_filename" id="kml_filename_show"></span>';
    if ( isset($args['desc']) ) {
      $html .= '&nbsp;<label for="'.$args['id'].'">'.$args['desc'].'</label>';
    }
    echo $html;
  }

  public function restrict_kmlmime_types($mime_types) {
    #$mime_types['kml'] = 'application/vnd.google-earth.kml+xml';
    #$mime_types['kmz'] = 'application/vnd.google-earth.kmz';
    $mime_types['kml'] = 'text/xml';
    $mime_types['kmz'] = 'application/zip';
    return $mime_types;
  }

  /**
   * Validation of settings
   *
   * @param [type] $input
   * @return void
   */
  public function validate_getlocations_lite_default_settings( $input ) {
    include_once(GETLOCATIONS_LITE_PLUGIN_DIR . 'includes/class-getlocations-lite-share.php');
    $share = new Getlocations_Lite_Share();

    $positions = array_keys($this->controlpositions);
    $position_fields = array('zoomcontrol_position','scale_position','layerscontrol_position');
    $available_maps = array_keys($this->map_providers);
    $txt_fields = array('map_provider', 'width', 'height', 'mapcenter');
    $default_settings_keys = array_keys($this->default_settings);
    $output = array();
    foreach( $input AS $key => $value ) {

      if ( ! preg_match("/^layers_/", $key) ) {
        if ( ! in_array($key, $default_settings_keys, TRUE) ) {
          continue;
        }
      }

      // sanitize text fields
      if ( in_array( $key, $txt_fields, TRUE ) ) {
        $output[$key] = sanitize_text_field($input[ $key ]);
      }
      else {
        $output[$key] = strip_tags( stripslashes( $input[ $key ] ) );
      }

      // positions
      if ( in_array( $key, $position_fields, TRUE ) ) {
        if ( ! in_array( $output[$key], $positions ) ) {
          // set to default
          $output[$key] = $this->default_settings[$key];
        }
      }
      // bool
      elseif ( in_array( $key, $this->bool, TRUE ) ) {
        if ( $output[$key] !== 1 && $output[$key] !== '1' && $output[$key] !== 0 && $output[$key] !== '0' ) {
          $output[$key] = $this->default_settings[$key];
        }
      }
      // available maps
      elseif ( $key == 'map_provider' ) {
        if (! in_array( $output[$key], $available_maps, TRUE ) ) {
          $output[$key] = $this->default_settings[$key];
        }
      }
      elseif ( $key == 'mapcenter' ) {
        if ( ! $mp = $share->latlon_check($output[$key]) ) {
          $output[$key] = $this->default_settings[$key];
        }
        else {
          $output[$key] = $mp;
        }
      }
      // zoom minzoom maxzoom. different maps have different maxzoom
      elseif ( $key == 'zoom' || $key == 'minzoom' || $key == 'maxzoom' ) {
        if ( $output[$key] < 0 || $output[$key] > 21 ) {
          $output[$key] = $this->default_settings[$key];
        }
      }
      // width height
      elseif ( $key == 'width' || $key == 'height' ) {
        if ( ! preg_match( "/px$/", $output[$key] ) && ! preg_match( "/em$/", $output[$key] ) && ! preg_match( "/in$/", $output[$key] ) && ! preg_match( "/%$/", $output[$key] ) ) {
          $output[$key] = $this->default_settings[$key];
        }
        elseif ( strlen($output[$key]) > 10 ) {
          $output[$key] = $this->default_settings[$key];
        }
      }
      // layers_
      elseif ( preg_match("/^layers_/", $key) ) {
        $tkey = preg_replace("/^layers_/", '', $key);
        if ( ! in_array($tkey, $available_maps, TRUE ) ) {
          unset($output[$key]);
        }
      }
    }
    return apply_filters( 'validate_getlocations_lite_default_settings', $output, $input );
  }

  /**
   * Validation of settings
   *
   * @param [type] $input
   * @return void
   */
  public function validate_getlocations_lite_map_keys( $input ) {
    $output = array();
    foreach( $input AS $key => $value ) {
      if( isset( $input[$key] ) ) {
        $output[$key] = strip_tags( stripslashes( $input[ $key ] ) );
      }
    }
    return apply_filters( 'validate_getlocations_lite_map_keys', $output, $input );
  }

  /**
   * Validation of settings
   *
   * @param [array] $input
   * @return array
   */
  public function validate_getlocations_lite_providers( $input ) {
    $output = array();
    $map_keys = get_option('getlocations_lite_map_keys');
    foreach( $input AS $key => $value ) {
      // we want to check that the providers have a key if required
      $tmpk = preg_replace("/^(.*)\..*$/", "$1", $key);
      $arr_map_keys = array_keys($map_keys);
      if ( in_array( $tmpk, $arr_map_keys, TRUE ) ) {
        if ( ! empty( $map_keys[$tmpk] ) ) {
          $output[$key] = strip_tags( stripslashes( $input[ $key ] ) );
        }
      }
      else {
        $output[$key] = strip_tags( stripslashes( $input[ $key ] ) );
      }
    }
    return apply_filters( 'validate_getlocations_lite_providers', $output, $input );
  }

  /**
   * Validation of settings
   *
   * @param [type] $input
   * @return void
   */
  public function validate_getlocations_lite_geocoders( $input ) {

    $output = array();
    foreach( $input as $key => $value ) {
      // look for _enable
      if ( preg_match("/_enable$/", $key) && $value == 1 ) {
        // split
        $a = explode('_', $key);
        $service = $a[0];
        // look up to get info about this service
        if ( isset( $this->default_geocoder_services[$service] ) ) {
          // is a key required
          if ( $this->default_geocoder_services[$service]['key'] ) {
            // neutrino
            if ( isset( $input[ $service . '_userId' ] ) && $input[ $service . '_userId' ] && isset( $input[ $service . '_apiKey' ] ) && $input[ $service . '_apiKey' ] ) {
              $input[ $service . '_enable' ] = 1;
            }
            else {
              if ( empty( $input[ $service . '_key' ] ) ) {
                #$input[ $service . '_enable' ] = 0;
                $input[ $key ] = 0;
              }
            }
          }
        }
        else {
          // this service does not exist so kill it altogether
          unset($input[ $key ]);
          continue;
        }
      }
        $output[ $key ] = strip_tags( stripslashes( $input[ $key ] ) );
    }
    return apply_filters( 'validate_getlocations_lite_geocoders', $output, $input );
  }

  /**
   * Validation of available plugins
   *
   * @param [array] $input
   * @return array
   */
  public function validate_getlocations_lite_plugins( $input ) {

    if (! is_array($input) ) {
      $input = array();
    }

    // array of plugins
    $plugins = array_keys( $this->default_leaflet_plugin_settings );

    $output = array();
    foreach( $input as $key => $value ) {
      $tmpkey = preg_replace("/^leaflet_plugin_/", '', $key);
      if ( ! in_array( $tmpkey, $plugins, TRUE ) ) {
        continue;
      }

      if ( $input[$key] !== 1 && $input[$key] !== '1' && $input[$key] !== 0 && $input[$key] !== '0' ) {
        $output[$key] = $this->default_leaflet_plugin_settings[$key];
      }
      else {
        $output[$key] = strip_tags( stripslashes( $input[ $key ] ) );
      }

    }
    return apply_filters( 'validate_getlocations_lite_plugins', $output, $input );
  }

  /**
   * Validation of plugins_settings
   *
   * @param [array] $input
   * @return array
   */
  public function validate_getlocations_lite_plugins_settings( $input ) {

    include_once(GETLOCATIONS_LITE_PLUGIN_DIR . 'includes/class-getlocations-lite-share.php');
    $share = new Getlocations_Lite_Share();

    if (! is_array($input) ) {
      $input = array();
    }
    $output = array();
    $positions = array_keys($this->controlpositions);
    $position_fields = array('fullscreen_position','mouseposition_position','magnifyingglass_position', 'graticule_position', 'terminator_position', 'geocoder_position');
    $txt_fields = array('fullscreen_label','fullscreen_cancel_label','mouseposition_emptystring','mouseposition_separator','mouseposition_prefix',
      'graticule_label','graticule_font','terminator_label','geocoder_placeholder','geocoder_label', 'geocoder_country', 'geocoder_service');
    $hex_color_fields = array('graticule_line_color','graticule_font_color','terminator_line_color','terminator_fill_color');
    $opacity_fields = array('graticule_line_opacity','terminator_line_opacity','terminator_fill_opacity');
    $integer_fields = array('mouseposition_numdigits','magnifyingglass_radius','magnifyingglass_offset','graticule_line_thickness','terminator_line_thickness');
    $plugins_bool = array_keys($this->plugins_bool);
    $plugins_bool_fields = array();
    foreach ( $plugins_bool AS $f ) {
      $plugins_bool_fields[] = $f . '_enable';
    }
    $plugins_bool_fields[] = 'fullscreen_forceseparatebutton';
    $plugins_bool_fields[] = 'mouseposition_display_dms';
    $plugins_bool_fields[] = 'graticule_initial_state';
    $plugins_bool_fields[] = 'graticule_linelabel';
    $country_codes = array_keys($share->get_countries_list());

    $geocoders = get_option('getlocations_lite_geocoders');
    // look for 'enable' in $geocoders
    $geocoder_services = array();
    foreach ( $geocoders AS $k => $v ) {
      if ( preg_match("/_enable$/", $k) && $v == 1 ) {
        $a = explode('_', $k);
        $geocoder_services[] = $a[0];
      }
    }

    // flatten the defaults
    $default_leaflet_plugin_settings = array();
    $plugins = array_keys( $this->default_leaflet_plugin_settings );
    foreach ( $plugins AS $plugin ) {
      foreach ( $this->default_leaflet_plugin_settings[$plugin] AS $pk => $pv ) {
        $default_leaflet_plugin_settings[$plugin . '_' . $pk] = $pv;
      }
    }
    // make a list of allowable input keys
    $default_leaflet_plugin_settings_keys = array_keys($default_leaflet_plugin_settings);

    // iterate over the inputs
    foreach( $input as $key => $value ) {

      // is this a valid entry? if not throw it out
      if ( ! in_array($key, $default_leaflet_plugin_settings_keys, TRUE ) ) {
        continue;
      }

      // sanitize text fields
      if ( in_array( $key, $txt_fields, TRUE ) || in_array($key, $opacity_fields, TRUE ) || in_array($key, $integer_fields, TRUE ) || in_array($key, $hex_color_fields, TRUE ) ) {
        $output[$key] = sanitize_text_field($input[ $key ]);
        // set a generic maximum length for textfields
        $fieldlength = 30;
        if ( strlen($output[$key]) > $fieldlength ) {
          $output[$key] = substr($output[$key], 0, $fieldlength);
        }
      }
      else {
        $output[$key] = strip_tags( stripslashes( $input[ $key ] ) );
      }

      // positions
      if ( in_array( $key, $position_fields, TRUE ) ) {
        if ( ! in_array( $output[$key], $positions, TRUE ) ) {
          // set to default
          $output[$key] = $default_leaflet_plugin_settings[$key];
        }
      }

      // bool
      elseif ( in_array( $key, $plugins_bool_fields, TRUE ) ) {
        if ( $output[$key] !== 1 && $output[$key] !== '1' && $output[$key] !== 0 && $output[$key] !== '0' ) {
          $output[$key] = $default_leaflet_plugin_settings[$key];
        }
      }

      // hex_color_fields
      elseif ( in_array($key, $hex_color_fields, TRUE ) ) {
        if ( ! sanitize_hex_color($output[$key]) ) {
          $output[$key] = $default_leaflet_plugin_settings[$key];
        }
      }

      // opacity_fields
      elseif ( in_array( $key, $opacity_fields, TRUE ) ) {
        if ( floatval($output[$key]) === 0 ) {
          $output[$key] = $default_leaflet_plugin_settings[$key];
        }
      }

      // integer_fields
      elseif ( in_array( $key, $integer_fields, TRUE ) ) {
        $tmp = intval($output[$key]);
        if ( $key == 'mouseposition_numdigits' ) {
          if ( $tmp < 1 || $tmp > 10 ) {
            $output[$key] = $default_leaflet_plugin_settings[$key];
          }
          else {
            $output[$key] = $tmp;
          }
        }
        elseif ( $key == 'magnifyingglass_radius' ) {
          if ( $tmp < 10 || $tmp > 1000 ) {
            $output[$key] = $default_leaflet_plugin_settings[$key];
          }
          else {
            $output[$key] = $tmp;
          }
        }
        elseif ( $key == 'magnifyingglass_offset' ) {
          if ( $tmp < 1 || $tmp > 19 ) {
            $output[$key] = $default_leaflet_plugin_settings[$key];
          }
          else {
            $output[$key] = $tmp;
          }
        }
        elseif ( $key == 'graticule_line_thickness' ) {
          if ( $tmp < 1 || $tmp > 20 ) {
            $output[$key] = $default_leaflet_plugin_settings[$key];
          }
          else {
            $output[$key] = $tmp;
          }
        }
        elseif ( $key == 'terminator_line_thickness' ) {
          if ( $tmp < 0 || $tmp > 20 ) {
            $output[$key] = $default_leaflet_plugin_settings[$key];
          }
          else {
            $output[$key] = $tmp;
          }
        }
      }
      elseif ( $key == 'geocoder_country' && ! empty($output[$key]) ) {
        if (! in_array($output[$key], $country_codes, TRUE ) ) {
          $output[$key] = $default_leaflet_plugin_settings[$key];
        }
      }
      elseif ( $key == 'geocoder_service' ) {
        if ( ! in_array( $output[$key], $geocoder_services, TRUE ) ) {
          $output[$key] = $default_leaflet_plugin_settings[$key];
        }
      }
    }
    return apply_filters( 'validate_getlocations_lite_plugins_settings', $output, $input );

  }

  /**
   * insert js into DOM via enqueue system
   * @param string $js
   *
   */
  private function admin_enqueue_inline($js) {
    wp_enqueue_script( $this->plugin_name . '-inline', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'admin/js/inline.js', array(), $this->version, TRUE );
    wp_add_inline_script( $this->plugin_name . '-inline', $js );
  }

  /**
   * allow themes to override templates
   *
   */
  private function process_template($file) {
    $themepath = get_stylesheet_directory();
    if ( file_exists( $themepath .'/'. GETLOCATIONS_LITE_PLUGIN_SLUG . '/admin/' . $file ) ) {
      $path = $themepath .'/'. GETLOCATIONS_LITE_PLUGIN_SLUG . '/admin/' . $file;
    }
    else {
      $path = GETLOCATIONS_LITE_PLUGIN_ADMIN_TEMPLATES_DIR . $file;
    }
    return $path;
  }

}

