<?php
/**
 * @file    leaflet_default_settings_section.php
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 */
  add_settings_section(
    $whichsection,                             // ID used to identify this section and with which to register options
    $whichtitle,                               // Title to be displayed on the administration page
    array( $this, $desc_callback ),            // Callback used to render the description of the section
    $whichpage                                 // Page on which to add this section of options
  );

  add_settings_field(
    'leaflet_map',                         // ID used to identify the field throughout the theme
    __( 'Map', 'getlocations-lite' ),      // The label to the left of the option interface element
    array( $this, 'map_element_callback'), // The name of the function responsible for rendering the option interface
    $whichpage,                            // The page on which this option will be displayed
    $whichsection,                         // The name of the section to which this field belongs
    array(                                 // The array of arguments to pass to the callback.
      'id'=>'leaflet_map',
      'desc' => __( 'The map.', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );

  add_settings_field(
    'map_provider',                            // ID used to identify the field throughout the theme
    __( 'Default map', 'getlocations-lite' ),  // The label to the left of the option interface element
    array( $this, 'select_element_callback'),  // The name of the function responsible for rendering the option interface
    $whichpage,                                // The page on which this option will be displayed
    $whichsection,                             // The name of the section to which this field belongs
    array(                                     // The array of arguments to pass to the callback.
      'id' => 'map_provider',
      'select_opts' => $maps,
      'opt' => $saveto
    )
  );
  add_settings_field(
    'width',                                 // ID used to identify the field throughout the theme
    __( 'Map width', 'getlocations-lite' ),  // The label to the left of the option interface element
    array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
    $whichpage,                              // The page on which this option will be displayed
    $whichsection,                           // The name of the section to which this field belongs
    array(                                   // The array of arguments to pass to the callback.
      'id' => 'width',
      'widthclass' => 'medium-text',
      'desc' => __( 'The width of a map, as a CSS length or percentage. <br>Examples: 50px, 5em, 2.5in, 95%', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'height',                                // ID used to identify the field throughout the theme
    __( 'Map height', 'getlocations-lite' ), // The label to the left of the option interface element
    array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
    $whichpage,                              // The page on which this option will be displayed
    $whichsection,                           // The name of the section to which this field belongs
    array(                                   // The array of arguments to pass to the callback.
      'id' => 'height',
      'widthclass' => 'medium-text',
      'desc' => __( 'The height of a map, as a CSS length or percentage. <br>Examples: 50px, 5em, 2.5in, 95%', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'mapcenter',                             // ID used to identify the field throughout the theme
    __( 'Map center', 'getlocations-lite' ), // The label to the left of the option interface element
    array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
    $whichpage,                              // The page on which this option will be displayed
    $whichsection,                           // The name of the section to which this field belongs
    array(                                   // The array of arguments to pass to the callback.
      'id' => 'mapcenter',
      'widthclass' => 'regular-text',
      'desc' => __('The center coordinates of a map, expressed as a decimal latitude and longitude, separated by a comma. <br>This must not be 0,0', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'zoom',                                    // ID used to identify the field throughout the theme
    __( 'Zoom', 'getlocations-lite' ),         // The label to the left of the option interface element
    array( $this, 'select_element_callback'),  // The name of the function responsible for rendering the option interface
    $whichpage,                                // The page on which this option will be displayed
    $whichsection,                             // The name of the section to which this field belongs
    array(                                     // The array of arguments to pass to the callback.
      'id' => 'zoom',
      'select_opts' => $this->zoomlevels,
      'opt' => $saveto
    )
  );
  add_settings_field(
    'minzoom',                                 // ID used to identify the field throughout the theme
    __( 'Minimum Zoom', 'getlocations-lite' ), // The label to the left of the option interface element
    array( $this, 'select_element_callback'),  // The name of the function responsible for rendering the option interface
    $whichpage,                                // The page on which this option will be displayed
    $whichsection,                             // The name of the section to which this field belongs
    array(                                     // The array of arguments to pass to the callback.
      'id' => 'minzoom',
      'select_opts' => $this->zoomlevels,
      'opt' => $saveto
    )
  );
  add_settings_field(
    'maxzoom',                                 // ID used to identify the field throughout the theme
    __( 'Maximum Zoom', 'getlocations-lite' ), // The label to the left of the option interface element
    array( $this, 'select_element_callback'),  // The name of the function responsible for rendering the option interface
    $whichpage,                                // The page on which this option will be displayed
    $whichsection,                             // The name of the section to which this field belongs
    array(                                     // The array of arguments to pass to the callback.
      'id' => 'maxzoom',
      'select_opts' => $this->zoomlevels,
      'opt' => $saveto
    )
  );
  add_settings_field(
    'zoomcontrol',                                // ID used to identify the field throughout the theme
    __( 'Zoom Control', 'getlocations-lite' ),    // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'zoomcontrol',
      'desc' => __( 'Enable zoom control on the map.', 'getlocations-lite' ),
      'class' => 'settings_section_top',
      'opt' => $saveto
    )
  );
  add_settings_field(
    'zoomcontrol_position',                                    // ID used to identify the field throughout the theme
    __( 'Position of the Zoom buttons', 'getlocations-lite' ), // The label to the left of the option interface element
    array( $this, 'select_element_callback'),                  // The name of the function responsible for rendering the option interface
    $whichpage,                                                // The page on which this option will be displayed
    $whichsection,                                             // The name of the section to which this field belongs
    array(                                                     // The array of arguments to pass to the callback.
      'id' => 'zoomcontrol_position',
      'select_opts' => $this->controlpositions,
      'default_opt' => 'topleft',
      'class' => 'leaflet_zoomControl',
      'opt' => $saveto
    )
  );
  add_settings_field(
    'scrollwheelzoom',                            // ID used to identify the field throughout the theme
    __( 'Scrollwheel', 'getlocations-lite' ),     // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'scrollwheelzoom',
      'desc' => __( 'Enable scrollwheel zooming.', 'getlocations-lite' ),
      'class' => 'settings_section_top',
      'opt' => $saveto
    )
  );
  add_settings_field(
    'dragging',                                   // ID used to identify the field throughout the theme
    __( 'Draggable', 'getlocations-lite' ),       // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'dragging',
      'desc' => __( 'Enable dragging the map.', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'doubleclickzoom',                            // ID used to identify the field throughout the theme
    __( 'DoubleClick Zoom', 'getlocations-lite' ),// The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'doubleclickzoom',
      'desc' => __( 'Enable DoubleClick Zooming.', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'boxzoom',                                    // ID used to identify the field throughout the theme
    __( 'Box Zoom', 'getlocations-lite' ),        // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'boxzoom',
      'desc' => __( 'Enable Box Zooming.', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'fitbounds',                                  // ID used to identify the field throughout the theme
    __( 'Fit bounds', 'getlocations-lite' ),      // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'fitbounds',
      'desc' => __( 'Enabling Fitbounds will show all the markers.', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'attributioncontrol',                             // ID used to identify the field throughout the theme
    __( 'Attribution Control', 'getlocations-lite' ), // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),       // The name of the function responsible for rendering the option interface
    $whichpage,                                       // The page on which this option will be displayed
    $whichsection,                                    // The name of the section to which this field belongs
    array(                                            // The array of arguments to pass to the callback.
      'id' => 'attributioncontrol',
      'desc' => __( 'Switch on attributions on the map.', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'keyboard',                                   // ID used to identify the field throughout the theme
    __( 'Keyboard', 'getlocations-lite' ),        // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'keyboard',
      'desc' => __( 'Allow keyboard zoom and pan on the map.', 'getlocations-lite' ),
      'opt' => $saveto
    )
  );
  add_settings_field(
    'scale',                                      // ID used to identify the field throughout the theme
    __( 'Scale Control', 'getlocations-lite' ),   // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'scale',
      'desc' => __( 'Show a scale in Metric/Imperial on the map.', 'getlocations-lite' ),
      'class' => 'settings_section_top',
      'opt' => $saveto
    )
  );
  add_settings_field(
    'scaleunit',                               // ID used to identify the field throughout the theme
    __( 'Scale Units', 'getlocations-lite' ),  // The label to the left of the option interface element
    array( $this, 'select_element_callback'),  // The name of the function responsible for rendering the option interface
    $whichpage,                                // The page on which this option will be displayed
    $whichsection,                             // The name of the section to which this field belongs
    array(                                     // The array of arguments to pass to the callback.
      'id' => 'scaleunit',
      'select_opts' => $this->scaleunits,
      'class' => 'leaflet_scale',
      'opt' => $saveto
    )
  );
  add_settings_field(
    'scale_position',                                           // ID used to identify the field throughout the theme
    __( 'Position of the Scale control', 'getlocations-lite' ), // The label to the left of the option interface element
    array( $this, 'select_element_callback'),                   // The name of the function responsible for rendering the option interface
    $whichpage,                                                 // The page on which this option will be displayed
    $whichsection,                                              // The name of the section to which this field belongs
    array(                                                      // The array of arguments to pass to the callback.
      'id' => 'scale_position',
      'select_opts' => $this->controlpositions,
      'default_opt' => 'bottomleft',
      'class' => 'leaflet_scale',
      'opt' => $saveto
    )
  );
  // layersControl
  add_settings_field(
    'layerscontrol',                              // ID used to identify the field throughout the theme
    __( 'Layers Control', 'getlocations-lite' ),  // The label to the left of the option interface element
    array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
    $whichpage,                                   // The page on which this option will be displayed
    $whichsection,                                // The name of the section to which this field belongs
    array(                                        // The array of arguments to pass to the callback.
      'id' => 'layerscontrol',
      'desc' => __( 'Enable layers control on the map.', 'getlocations-lite' ),
      'class' => 'settings_section_top',
      'opt' => $saveto
    )
  );
  add_settings_field(
    'layerscontrol_position',                                   // ID used to identify the field throughout the theme
    __( 'Position of the Layers button', 'getlocations-lite' ), // The label to the left of the option interface element
    array( $this, 'select_element_callback'),                   // The name of the function responsible for rendering the option interface
    $whichpage,                                                 // The page on which this option will be displayed
    $whichsection,                                              // The name of the section to which this field belongs
    array(                                                      // The array of arguments to pass to the callback.
      'id' => 'layerscontrol_position',
      'select_opts' => $this->controlpositions,
      'class' => 'leaflet_layersControl',
      'opt' => $saveto
    )
  );
  // minimap leaflet_plugin_minimap getlocations_lite_plugins
  $getlocations_lite_plugins = get_option('getlocations_lite_plugins');
  if ( isset($getlocations_lite_plugins['leaflet_plugin_minimap']) && $getlocations_lite_plugins['leaflet_plugin_minimap'] ) {
    // checkbox
    add_settings_field(
      'minimap',                                     // ID used to identify the field throughout the theme
      __( 'Minimap enhanced', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),    // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'minimap',
        'desc' => __( 'Use Minimap enhancement on the map.', 'getlocations-lite' ),
        'class' => 'leaflet_layersControl',
        'opt' => $saveto
      )
    );
  }

  // add a list of maps
  $default_settings = get_option('getlocations_lite_default_settings');
  $default_map = $default_settings['map_provider'];
  $selected_providers = get_option('getlocations_lite_providers');
  $providers_list = array_keys($selected_providers);
  $map_key_data = get_option('getlocations_lite_map_keys');
  if ( isset($map_key_data['Thunderforest']) ) {
    $thunderforest_key = $map_key_data['Thunderforest'];
  }
  #if ( isset($map_key_data['Openweathermap']) ) {
  #  $openweathermap_key = $map_key_data['Openweathermap'];
  #}

  include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/providers.php');

  // only show the maps that have a key or do not need one
  foreach ($default_leaflet_providers_data AS $k => $v) {
    $go = TRUE;
    if (! empty($v['options']) ) {
      $go = FALSE;
    }

    if ( $v['group'] == 'Thunderforest' ) {
      if (! empty($v['options']['apikey']) && $v['options']['apikey'] == $thunderforest_key ) {
        $go = TRUE;
      }
      else{
        $go = FALSE;
      }
    }
    elseif ( $v['group'] == 'OpenWeatherMap' ) {
      if (! empty($v['options']['apiKey']) && $v['options']['apiKey'] == $openweathermap_key ) {
        $go = TRUE;
      }
      else{
        $go = FALSE;
      }
    }

    // must be in list
    if (! in_array($k, $providers_list) ) {
      $go = FALSE;
    }

    if ($go) {
      $label = __('Include', 'getlocations-lite') . ': ' . $v['label'];
      $key = 'layers_' . $k;
      $optsarr = array(
        'id' => $key,
          'desc' => __('Type', 'getlocations-lite') . ': ' . ucfirst($v['type']),
          'class' => 'leaflet_layersControl',
          'opt' => $saveto
      );
      if ( $default_map == $k ) {
        $optsarr['default_opt'] = '1';
      }
      add_settings_field(
        $key,                                       // ID used to identify the field throughout the theme
        $label,                                     // The label to the left of the option interface element
        array( $this, 'checkbox_element_callback'), // The name of the function responsible for rendering the option interface
        $whichpage,                                 // The page on which this option will be displayed
        $whichsection,                              // The name of the section to which this field belongs
        $optsarr                                    // The array of arguments to pass to the callback.
      );
    }
  }
