<?php
/**
 * @file    leaflet_geocoders_section.php
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 */
  add_settings_section(
    $whichsection,                               // ID used to identify this section and with which to register options
    __( 'Leaflet Geocoder Services', 'getlocations-lite' ), // Title to be displayed on the administration page
    array( $this, 'leaflet_geocoders_callback'),  // Callback used to render the description of the section
    $whichpage                                   // Page on which to add this section of options
  );

  $plugins = get_option('getlocations_lite_plugins');
  if ( isset($plugins['leaflet_plugin_geocoder']) && $plugins['leaflet_plugin_geocoder'] ) {
    // nominatim(options) default
    add_settings_field(
      'nominatim_enable',                                // ID used to identify the field throughout the theme
      __( 'Enable Nominatim Geocoding', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'nominatim_enable',
        'desc' => __( 'This service does not require a key.', 'getlocations-lite' ),
        'default_opt' => $this->default_geocoder_services['nominatim']['enable'],
        'opt' => $saveto
      )
    );

    // other search engines
    // arcgis(accessToken, options) WORKS
    add_settings_field(
      'arcgis_enable',                                // ID used to identify the field throughout the theme
      __( 'Enable ArcGis Geocoding', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'arcgis_enable',
        'desc' => __( 'This service requires a key.', 'getlocations-lite' ),
        'default_opt' => $this->default_geocoder_services['arcgis']['enable'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'arcgis_key',                                          // ID used to identify the field throughout the theme
      __( 'ArcGis geocoder API key', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'text_element_callback'),                   // The name of the function responsible for rendering the option interface
      $whichpage,                                               // The page on which this option will be displayed
      $whichsection,                                            // The name of the section to which this field belongs
      array(                                                    // The array of arguments to pass to the callback.
        'id'=>'arcgis_key',
        'widthclass' => 'regular-text',
        'desc' => __('Get your key <a href="https://developers.arcgis.com/" target="_blank" ><b>here</b></a>.', 'getlocations-lite'),
        'class' => 'geocoder_arcgis',
        'opt' => $saveto
      )
    );

    // bing requires key
    // google(key, options)
    add_settings_field(
      'google_enable',                                // ID used to identify the field throughout the theme
      __( 'Enable Google Geocoding', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'google_enable',
        'desc' => __( 'This service requires a key.', 'getlocations-lite' ),
        'default_opt' => $this->default_geocoder_services['google']['enable'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'google_key',                                          // ID used to identify the field throughout the theme
      __( 'Google key', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'text_element_callback'),                   // The name of the function responsible for rendering the option interface
      $whichpage,                                               // The page on which this option will be displayed
      $whichsection,                                            // The name of the section to which this field belongs
      array(                                                    // The array of arguments to pass to the callback.
        'id'=>'google_key',
        'widthclass' => 'regular-text',
        'desc' => __('Get your key <a href="https://google.com/" target="_blank" ><b>here</b></a>.', 'getlocations-lite'),
        'class' => 'geocoder_google',
        'opt' => $saveto
      )
    );

    // here requires app_id and app_code
    // latLng returns place from lat lng
    // mapbox(accessToken, options)
    add_settings_field(
      'mapbox_enable',                                // ID used to identify the field throughout the theme
      __( 'Enable Mapbox Geocoding', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'mapbox_enable',
        'desc' => __( 'This service requires a key.', 'getlocations-lite' ),
        'default_opt' => $this->default_geocoder_services['mapbox']['enable'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'mapbox_key',                                          // ID used to identify the field throughout the theme
      __( 'Mapbox geocoder API key', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'text_element_callback'),                   // The name of the function responsible for rendering the option interface
      $whichpage,                                               // The page on which this option will be displayed
      $whichsection,                                            // The name of the section to which this field belongs
      array(                                                    // The array of arguments to pass to the callback.
        'id'=>'mapbox_key',
        'widthclass' => 'regular-text',
        'desc' => __('Get your key <a href="https://account.mapbox.com/" target="_blank" ><b>here</b></a>.', 'getlocations-lite'),
        'class' => 'geocoder_mapbox',
        'opt' => $saveto
      )
    );

    // mapQuest(key, options)
    // neutrino userid and apikey neutrino(accessToken)
    add_settings_field(
      'neutrino_enable',                                // ID used to identify the field throughout the theme
      __( 'Enable Neutrino Geocoding', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'neutrino_enable',
        'desc' => __( 'This service requires a key.', 'getlocations-lite' ),
        'default_opt' => $this->default_geocoder_services['neutrino']['enable'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'neutrino_userId',                                          // ID used to identify the field throughout the theme
      __( 'Neutrino geocoder User ID', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'text_element_callback'),                   // The name of the function responsible for rendering the option interface
      $whichpage,                                               // The page on which this option will be displayed
      $whichsection,                                            // The name of the section to which this field belongs
      array(                                                    // The array of arguments to pass to the callback.
        'id'=>'neutrino_userId',
        'desc' => __('Sign on <a href="https://www.neutrinoapi.com/" target="_blank" ><b>here</b></a>.', 'getlocations-lite'),
        'widthclass' => 'regular-text',
        'class' => 'geocoder_neutrino',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'neutrino_apiKey',                                          // ID used to identify the field throughout the theme
      __( 'Neutrino geocoder API key', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'text_element_callback'),                   // The name of the function responsible for rendering the option interface
      $whichpage,                                               // The page on which this option will be displayed
      $whichsection,                                            // The name of the section to which this field belongs
      array(                                                    // The array of arguments to pass to the callback.
        'id'=>'neutrino_apiKey',
        'widthclass' => 'regular-text',
        'class' => 'geocoder_neutrino',
        'opt' => $saveto
      )
    );

    // openLocationCode(options) ?
    // opencage(apiKey) ?
    add_settings_field(
      'opencage_enable',                                // ID used to identify the field throughout the theme
      __( 'Enable OpenCage Geocoding', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'opencage_enable',
        'desc' => __( 'This service requires a key.', 'getlocations-lite' ),
        'default_opt' => $this->default_geocoder_services['opencage']['enable'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'opencage_key',                                          // ID used to identify the field throughout the theme
      __( 'OpenCage geocoder API key', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'text_element_callback'),                   // The name of the function responsible for rendering the option interface
      $whichpage,                                               // The page on which this option will be displayed
      $whichsection,                                            // The name of the section to which this field belongs
      array(                                                    // The array of arguments to pass to the callback.
        'id'=>'opencage_key',
        'widthclass' => 'regular-text',
        'desc' => __('Get your key <a href="https://opencagedata.com/" target="_blank" ><b>here</b></a>.', 'getlocations-lite'),
        'class' => 'geocoder_opencage',
        'opt' => $saveto
      )
    );

    //pelias(apiKey, options)

    // photon(options) WORKS, no country limiter AFAICT
    add_settings_field(
      'photon_enable',                                // ID used to identify the field throughout the theme
      __( 'Enable Photon Geocoding', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'photon_enable',
        'desc' => __( 'This service does not require a key.', 'getlocations-lite' ),
        'default_opt' => $this->default_geocoder_services['photon']['enable'],
        'opt' => $saveto
      )
    );
    // gopts.geocoder = L.Control.Geocoder.photon();

    // what3words(accessToken)
    add_settings_field(
      'what3words_enable',                                // ID used to identify the field throughout the theme
      __( 'Enable what3words Geocoding', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'what3words_enable',
        'desc' => __( 'This service requires an access token.', 'getlocations-lite' ),
        'default_opt' => $this->default_geocoder_services['arcgis']['enable'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'what3words_key',                                          // ID used to identify the field throughout the theme
      __( 'what3words geocoder Access Token', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'text_element_callback'),                   // The name of the function responsible for rendering the option interface
      $whichpage,                                               // The page on which this option will be displayed
      $whichsection,                                            // The name of the section to which this field belongs
      array(                                                    // The array of arguments to pass to the callback.
        'id'=>'what3words_key',
        'widthclass' => 'regular-text',
        'desc' => __('Get your access token <a href="https://accounts.what3words.com/en/login" target="_blank" ><b>here</b></a>.', 'getlocations-lite'),
        'class' => 'geocoder_what3words',
        'opt' => $saveto
      )
    );

  }
  else {
    // geocoder plugin is not enable but set the default anyway
    add_settings_field(
      'nominatim_enable',                                // ID used to identify the field throughout the theme
      '',     // The label to the left of the option interface element
      array( $this, 'hidden_element_callback'),     // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'nominatim_enable',
        'default_opt' => $this->default_geocoder_services['nominatim']['enable'],
        'opt' => $saveto
      )
    );

  }
