<?php
/**
 * @file    leaflet_plugins_settings_section.php
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 * This presents the per-map settings for any enabled leaflet_plugins
 */
  add_settings_section(
    $whichsection,                                        // ID used to identify this section and with which to register options
    __( 'Leaflet Plugin Settings', 'getlocations-lite' ), // Title to be displayed on the administration page
    array( $this, 'leaflet_plugin_settings_callback'),    // Callback used to render the description of the section
    $whichpage                                            // Page on which to add this section of options
  );

  // only show plugins which have been enabled
  $plugins = $this->map_plugins;

  // leaflet_plugin_fullscreen
  if ( isset($plugins['leaflet_plugin_fullscreen']) && $plugins['leaflet_plugin_fullscreen'] ) {
    add_settings_field(
      'fullscreen_enable',                            // ID used to identify the field throughout the theme
      __( 'Enable Fullscreen', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),     // The name of the function responsible for rendering the option interface
      $whichpage,                                     // The page on which this option will be displayed
      $whichsection,                                  // The name of the section to which this field belongs
      array(                                          // The array of arguments to pass to the callback.
        'id' => 'fullscreen_enable',
        'desc' => __( 'Enable Fullscreen control on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['fullscreen']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'fullscreen_position',                                          // ID used to identify the field throughout the theme
      __( 'Position of the Fullscreen button', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                       // The name of the function responsible for rendering the option interface
      $whichpage,                                                     // The page on which this option will be displayed
      $whichsection,                                                  // The name of the section to which this field belongs
      array(                                                          // The array of arguments to pass to the callback.
        'id' => 'fullscreen_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['fullscreen']['position'],
        'class' => 'leaflet_fullscreen',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'fullscreen_label',                      // ID used to identify the field throughout the theme
      __( 'Label', 'getlocations-lite' ),      // The label to the left of the option interface element
      array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
      $whichpage,                              // The page on which this option will be displayed
      $whichsection,                           // The name of the section to which this field belongs
      array(                                   // The array of arguments to pass to the callback.
        'id' => 'fullscreen_label',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_fullscreen',
        'default_opt' => $this->default_leaflet_plugin_settings['fullscreen']['label'],
        'desc' => __('Leave Empty for the default', 'getlocations-lite' ),
        'opt' => $saveto
      )
    );
    add_settings_field(
      'fullscreen_cancel_label',                // ID used to identify the field throughout the theme
      __( 'Close Label', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                               // The page on which this option will be displayed
      $whichsection,                            // The name of the section to which this field belongs
      array(                                    // The array of arguments to pass to the callback.
        'id' => 'fullscreen_cancel_label',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_fullscreen',
        'default_opt' => $this->default_leaflet_plugin_settings['fullscreen']['cancel_label'],
        'desc' => __('Leave Empty for the default', 'getlocations-lite' ),
        'opt' => $saveto
      )
    );
    add_settings_field(
      'fullscreen_forceseparatebutton',                  // ID used to identify the field throughout the theme
      __( 'Keep button separate', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                        // The page on which this option will be displayed
      $whichsection,                                     // The name of the section to which this field belongs
      array(                                             // The array of arguments to pass to the callback.
        'id' => 'fullscreen_forceseparatebutton',
        'desc' => __( 'Keep button separate from zoom button.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['fullscreen']['forceseparatebutton'],
        'class' => 'leaflet_fullscreen',
        'opt' => $saveto
      )
    );

  }

  // leaflet_plugin_mouseposition
  if ( isset($plugins['leaflet_plugin_mouseposition']) && $plugins['leaflet_plugin_mouseposition'] ) {

    add_settings_field(
      'mouseposition_enable',                            // ID used to identify the field throughout the theme
      __( 'Enable Mouseposition', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                        // The page on which this option will be displayed
      $whichsection,                                     // The name of the section to which this field belongs
      array(                                             // The array of arguments to pass to the callback.
        'id' => 'mouseposition_enable',
        'desc' => __( 'Enable Mouseposition control on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['mouseposition']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'mouseposition_position',                                    // ID used to identify the field throughout the theme
      __( 'Position of Mouseposition data', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                    // The name of the function responsible for rendering the option interface
      $whichpage,                                                  // The page on which this option will be displayed
      $whichsection,                                               // The name of the section to which this field belongs
      array(                                                       // The array of arguments to pass to the callback.
        'id' => 'mouseposition_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['mouseposition']['position'],
        'class' => 'leaflet_mouseposition',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'mouseposition_display_dms',                                 // ID used to identify the field throughout the theme
      __( 'Show Latitude/Longitude in DMS', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),                  // The name of the function responsible for rendering the option interface
      $whichpage,                                                  // The page on which this option will be displayed
      $whichsection,                                               // The name of the section to which this field belongs
      array(                                                       // The array of arguments to pass to the callback.
        'id' => 'mouseposition_display_dms',
        'desc' => __( 'Show Latitude/Longitude in Degrees, minutes, seconds.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['mouseposition']['display_dms'],
        'class' => 'leaflet_mouseposition',
        'opt' => $saveto
      )
    );
    # emptystring
    add_settings_field(
      'mouseposition_emptystring',                       // ID used to identify the field throughout the theme
      __( 'Default empty string', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),            // The name of the function responsible for rendering the option interface
      $whichpage,                                        // The page on which this option will be displayed
      $whichsection,                                     // The name of the section to which this field belongs
      array(                                             // The array of arguments to pass to the callback.
        'id' => 'mouseposition_emptystring',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_mouseposition',
        'default_opt' => $this->default_leaflet_plugin_settings['mouseposition']['emptystring'],
        'opt' => $saveto
      )
    );
    # separator
    add_settings_field(
      'mouseposition_separator',              // ID used to identify the field throughout the theme
      __( 'Separator', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'), // The name of the function responsible for rendering the option interface
      $whichpage,                             // The page on which this option will be displayed
      $whichsection,                          // The name of the section to which this field belongs
      array(                                  // The array of arguments to pass to the callback.
        'id' => 'mouseposition_separator',
        'widthclass' => 'small-text',
        'class' => 'leaflet_mouseposition',
        'default_opt' => $this->default_leaflet_plugin_settings['mouseposition']['separator'],
        'opt' => $saveto
      )
    );
    # lngfirst
    add_settings_field(
      'mouseposition_lngfirst',                                    // ID used to identify the field throughout the theme
      __( 'Show Longitude before Latitude', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),                  // The name of the function responsible for rendering the option interface
      $whichpage,                                                  // The page on which this option will be displayed
      $whichsection,                                               // The name of the section to which this field belongs
      array(                                                       // The array of arguments to pass to the callback.
        'id' => 'mouseposition_lngfirst',
        'default_opt' => $this->default_leaflet_plugin_settings['mouseposition']['lngfirst'],
        'class' => 'leaflet_mouseposition',
        'opt' => $saveto
      )
    );
    # numdigits
    add_settings_field(
      'mouseposition_numdigits',                     // ID used to identify the field throughout the theme
      __( 'Number of digits', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'mouseposition_numdigits',
        'widthclass' => 'small-text',
        'class' => 'leaflet_mouseposition',
        'default_opt' => $this->default_leaflet_plugin_settings['mouseposition']['numdigits'],
        'opt' => $saveto
      )
    );
    # prefix
    add_settings_field(
      'mouseposition_prefix',                        // ID used to identify the field throughout the theme
      __( 'Prefixed string', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'text_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'mouseposition_prefix',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_mouseposition',
        'default_opt' => $this->default_leaflet_plugin_settings['mouseposition']['prefix'],
        'opt' => $saveto
      )
    );
  }
  // leaflet_plugin_pancontrol
  if ( isset($plugins['leaflet_plugin_pancontrol']) && $plugins['leaflet_plugin_pancontrol'] ) {
    add_settings_field(
      'pancontrol_enable',                            // ID used to identify the field throughout the theme
      __( 'Enable Pancontrol', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),     // The name of the function responsible for rendering the option interface
      $whichpage,                                     // The page on which this option will be displayed
      $whichsection,                                  // The name of the section to which this field belongs
      array(                                          // The array of arguments to pass to the callback.
        'id' => 'pancontrol_enable',
        'desc' => __( 'Enable Pan control on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['pancontrol']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'pancontrol_position',                                    // ID used to identify the field throughout the theme
      __( 'Position of the Pan buttons', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                 // The name of the function responsible for rendering the option interface
      $whichpage,                                               // The page on which this option will be displayed
      $whichsection,                                            // The name of the section to which this field belongs
      array(                                                    // The array of arguments to pass to the callback.
        'id' => 'pancontrol_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['pancontrol']['position'],
        'class' => 'leaflet_pancontrol',
        'opt' => $saveto
      )
    );
  }
  // leaflet_plugin_zoomslider
  if ( isset($plugins['leaflet_plugin_zoomslider']) && $plugins['leaflet_plugin_zoomslider'] ) {
    add_settings_field(
      'zoomslider_enable',                            // ID used to identify the field throughout the theme
      __( 'Enable Zoomslider', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),     // The name of the function responsible for rendering the option interface
      $whichpage,                                     // The page on which this option will be displayed
      $whichsection,                                  // The name of the section to which this field belongs
      array(                                          // The array of arguments to pass to the callback.
        'id' => 'zoomslider_enable',
        'desc' => __( 'Enable Zoomslider control on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['zoomslider']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'zoomslider_position',                                   // ID used to identify the field throughout the theme
      __( 'Position of the Zoomslider', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                // The name of the function responsible for rendering the option interface
      $whichpage,                                              // The page on which this option will be displayed
      $whichsection,                                           // The name of the section to which this field belongs
      array(                                                   // The array of arguments to pass to the callback.
        'id' => 'zoomslider_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['zoomslider']['position'],
        'class' => 'leaflet_zoomslider',
        'opt' => $saveto
      )
    );
  }
  // leaflet_plugin_magnifyingglass
  if ( isset($plugins['leaflet_plugin_magnifyingglass']) && $plugins['leaflet_plugin_magnifyingglass'] ) {
    add_settings_field(
      'magnifyingglass_enable',                      // ID used to identify the field throughout the theme
      __( 'Enable Magnifier', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),    // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'magnifyingglass_enable',
        'desc' => __( 'Enable Magnifier control on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['magnifyingglass']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'magnifyingglass_position',                              // ID used to identify the field throughout the theme
      __( 'Position of the Magnifier', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                // The name of the function responsible for rendering the option interface
      $whichpage,                                              // The page on which this option will be displayed
      $whichsection,                                           // The name of the section to which this field belongs
      array(                                                   // The array of arguments to pass to the callback.
        'id' => 'magnifyingglass_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['magnifyingglass']['position'],
        'class' => 'leaflet_magnifyingglass',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'magnifyingglass_radius',                      // ID used to identify the field throughout the theme
      __( 'Magnifier radius', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'magnifyingglass_radius',
        'widthclass' => 'small-text',
        'class' => 'leaflet_magnifyingglass',
        'default_opt' => $this->default_leaflet_plugin_settings['magnifyingglass']['radius'],
        'desc' => __( 'The Radius of the magnifying glass', 'getlocations-lite' ),
        'opt' => $saveto
      )
    );
    add_settings_field(
      'magnifyingglass_offset',                      // ID used to identify the field throughout the theme
      __( 'Magnifier offset', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'magnifyingglass_offset',
        'widthclass' => 'small-text',
        'class' => 'leaflet_magnifyingglass',
        'default_opt' => $this->default_leaflet_plugin_settings['magnifyingglass']['offset'],
        'desc' => __( 'The zoom offset of the magnifying glass', 'getlocations-lite' ),
        'opt' => $saveto
      )
    );
    add_settings_field(
      'magnifyingglass_forceseparatebutton',             // ID used to identify the field throughout the theme
      __( 'Keep button separate', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                        // The page on which this option will be displayed
      $whichsection,                                     // The name of the section to which this field belongs
      array(                                             // The array of arguments to pass to the callback.
        'id' => 'magnifyingglass_forceseparatebutton',
        'desc' => __( 'Keep button separate from zoom button.', 'getlocations-pro' ),
        'default_opt' => $this->default_leaflet_plugin_settings['magnifyingglass']['forceseparatebutton'],
        'class' => 'leaflet_magnifyingglass',
        'opt' => $saveto
      )
    );
  }

  // leaflet_plugin_graticule
  if ( isset($plugins['leaflet_plugin_graticule']) && $plugins['leaflet_plugin_graticule'] ) {
    add_settings_field(
      'graticule_enable',                            // ID used to identify the field throughout the theme
      __( 'Enable Graticule', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),    // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'graticule_enable',
        'desc' => __( 'Enable Graticule on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_position',                                           // ID used to identify the field throughout the theme
      __( 'Position of the Graticule button', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                       // The name of the function responsible for rendering the option interface
      $whichpage,                                                     // The page on which this option will be displayed
      $whichsection,                                                  // The name of the section to which this field belongs
      array(                                                          // The array of arguments to pass to the callback.
        'id' => 'graticule_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['position'],
        'class' => 'leaflet_graticule',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_line_color',                  // ID used to identify the field throughout the theme
      __( 'Line color', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
      $whichpage,                              // The page on which this option will be displayed
      $whichsection,                           // The name of the section to which this field belongs
      array(                                   // The array of arguments to pass to the callback.
        'id' => 'graticule_line_color',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_graticule',
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['line_color'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_line_opacity',                  // ID used to identify the field throughout the theme
      __( 'Line opacity', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),    // The name of the function responsible for rendering the option interface
      $whichpage,                                // The page on which this option will be displayed
      $whichsection,                             // The name of the section to which this field belongs
      array(                                     // The array of arguments to pass to the callback.
        'id' => 'graticule_line_opacity',
        'widthclass' => 'small-text',
        'class' => 'leaflet_graticule',
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['line_opacity'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_line_thickness',                  // ID used to identify the field throughout the theme
      __( 'Line thickness', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),      // The name of the function responsible for rendering the option interface
      $whichpage,                                  // The page on which this option will be displayed
      $whichsection,                               // The name of the section to which this field belongs
      array(                                       // The array of arguments to pass to the callback.
        'id' => 'graticule_line_thickness',
        'widthclass' => 'small-text',
        'class' => 'leaflet_graticule',
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['line_thickness'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_initial_state',                            // ID used to identify the field throughout the theme
      __( 'Initial State Graticule', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),           // The name of the function responsible for rendering the option interface
      $whichpage,                                           // The page on which this option will be displayed
      $whichsection,                                        // The name of the section to which this field belongs
      array(                                                // The array of arguments to pass to the callback.
        'id' => 'graticule_initial_state',
        'desc' => __( 'Initial state of graticule, on/off.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['initial_state'],
        'class' => 'leaflet_graticule',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_label',                       // ID used to identify the field throughout the theme
      __( 'Label', 'getlocations-lite' ),      // The label to the left of the option interface element
      array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
      $whichpage,                              // The page on which this option will be displayed
      $whichsection,                           // The name of the section to which this field belongs
      array(                                   // The array of arguments to pass to the callback.
        'id' => 'graticule_label',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_graticule',
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['label'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_linelabel',                         // ID used to identify the field throughout the theme
      __( 'Show Line Labels', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),    // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'graticule_linelabel',
        'desc' => __( 'Line Labels on/off.', 'getlocations-lite' ),
        #'default_opt' => $default_leaflet_plugin_settings['graticule']['linelabel'],
        'class' => 'leaflet_graticule',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_font',                             // ID used to identify the field throughout the theme
      __( 'Line label font', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),       // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'graticule_font',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_graticule',
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['font'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_font_color',                  // ID used to identify the field throughout the theme
      __( 'Font color', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
      $whichpage,                              // The page on which this option will be displayed
      $whichsection,                           // The name of the section to which this field belongs
      array(                                   // The array of arguments to pass to the callback.
        'id' => 'graticule_font_color',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_graticule',
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['font_color'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'graticule_forceseparatebutton',                   // ID used to identify the field throughout the theme
      __( 'Keep button separate', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                        // The page on which this option will be displayed
      $whichsection,                                     // The name of the section to which this field belongs
      array(                                             // The array of arguments to pass to the callback.
        'id' => 'graticule_forceseparatebutton',
        'desc' => __( 'Keep button separate from zoom button.', 'getlocations-pro' ),
        'default_opt' => $this->default_leaflet_plugin_settings['graticule']['forceseparatebutton'],
        'class' => 'leaflet_graticule',
        'opt' => $saveto
      )
    );
  }

  // leaflet_plugin_markercluster
  if ( isset($plugins['leaflet_plugin_markercluster']) && $plugins['leaflet_plugin_markercluster'] ) {
    add_settings_field(
      'markercluster_enable',                               // ID used to identify the field throughout the theme
      __( 'Enable Marker Clusterer', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),           // The name of the function responsible for rendering the option interface
      $whichpage,                                           // The page on which this option will be displayed
      $whichsection,                                        // The name of the section to which this field belongs
      array(                                                // The array of arguments to pass to the callback.
        'id' => 'markercluster_enable',
        'desc' => __( 'Enable Marker clusterer on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['markercluster']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
  }
  // leaflet_plugin_terminator
  if ( isset($plugins['leaflet_plugin_terminator']) && $plugins['leaflet_plugin_terminator'] ) {
    add_settings_field(
      'terminator_enable',                           // ID used to identify the field throughout the theme
      __( 'Enable Day/Night', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),    // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'terminator_enable',
        'desc' => __( 'Enable Terminator control on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_position',                                         // ID used to identify the field throughout the theme
      __( 'Position of the Day/Night button', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                      // The name of the function responsible for rendering the option interface
      $whichpage,                                                    // The page on which this option will be displayed
      $whichsection,                                                 // The name of the section to which this field belongs
      array(                                                         // The array of arguments to pass to the callback.
        'id' => 'terminator_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['position'],
        'class' => 'leaflet_terminator',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_line_color',                 // ID used to identify the field throughout the theme
      __( 'Line color', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
      $whichpage,                              // The page on which this option will be displayed
      $whichsection,                           // The name of the section to which this field belongs
      array(                                   // The array of arguments to pass to the callback.
        'id' => 'terminator_line_color',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_terminator',
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['line_color'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_line_opacity',                 // ID used to identify the field throughout the theme
      __( 'Line opacity', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),    // The name of the function responsible for rendering the option interface
      $whichpage,                                // The page on which this option will be displayed
      $whichsection,                             // The name of the section to which this field belongs
      array(                                     // The array of arguments to pass to the callback.
        'id' => 'terminator_line_opacity',
        'widthclass' => 'small-text',
        'class' => 'leaflet_terminator',
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['line_opacity'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_line_thickness',                 // ID used to identify the field throughout the theme
      __( 'Line thickness', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),      // The name of the function responsible for rendering the option interface
      $whichpage,                                  // The page on which this option will be displayed
      $whichsection,                               // The name of the section to which this field belongs
      array(                                       // The array of arguments to pass to the callback.
        'id' => 'terminator_line_thickness',
        'widthclass' => 'small-text',
        'class' => 'leaflet_terminator',
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['line_thickness'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_fill_color',                 // ID used to identify the field throughout the theme
      __( 'Fill color', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
      $whichpage,                              // The page on which this option will be displayed
      $whichsection,                           // The name of the section to which this field belongs
      array(                                   // The array of arguments to pass to the callback.
        'id' => 'terminator_fill_color',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_terminator',
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['fill_color'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_fill_opacity',                 // ID used to identify the field throughout the theme
      __( 'Fill opacity', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),    // The name of the function responsible for rendering the option interface
      $whichpage,                                // The page on which this option will be displayed
      $whichsection,                             // The name of the section to which this field belongs
      array(                                     // The array of arguments to pass to the callback.
        'id' => 'terminator_fill_opacity',
        'widthclass' => 'small-text',
        'class' => 'leaflet_terminator',
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['fill_opacity'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_initial_state',                           // ID used to identify the field throughout the theme
      __( 'Initial State Day/Night', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),           // The name of the function responsible for rendering the option interface
      $whichpage,                                           // The page on which this option will be displayed
      $whichsection,                                        // The name of the section to which this field belongs
      array(                                                // The array of arguments to pass to the callback.
        'id' => 'terminator_initial_state',
        'desc' => __( 'Initial state, on/off.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['initial_state'],
        'class' => 'leaflet_terminator',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_label',                      // ID used to identify the field throughout the theme
      __( 'Label', 'getlocations-lite' ),      // The label to the left of the option interface element
      array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
      $whichpage,                              // The page on which this option will be displayed
      $whichsection,                           // The name of the section to which this field belongs
      array(                                   // The array of arguments to pass to the callback.
        'id' => 'terminator_label',
        'widthclass' => 'regular-text',
        'class' => 'leaflet_terminator',
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['label'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'terminator_forceseparatebutton',                  // ID used to identify the field throughout the theme
      __( 'Keep button separate', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),        // The name of the function responsible for rendering the option interface
      $whichpage,                                        // The page on which this option will be displayed
      $whichsection,                                     // The name of the section to which this field belongs
      array(                                             // The array of arguments to pass to the callback.
        'id' => 'terminator_forceseparatebutton',
        'desc' => __( 'Keep button separate from zoom button.', 'getlocations-pro' ),
        'default_opt' => $this->default_leaflet_plugin_settings['terminator']['forceseparatebutton'],
        'class' => 'leaflet_terminator',
        'opt' => $saveto
      )
    );
  }
  // leaflet_plugin_geocoder
  if ( isset($plugins['leaflet_plugin_geocoder']) && $plugins['leaflet_plugin_geocoder'] ) {
    add_settings_field(
      'geocoder_enable',                            // ID used to identify the field throughout the theme
      __( 'Enable Geocoder', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'geocoder_enable',
        'desc' => __( 'Enable Geocoder on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['geocoder']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );

    $geocoders = get_option('getlocations_lite_geocoders');
    // look for 'enable' in $geocoders
    $dd = array();
    foreach ( $geocoders AS $k => $v ) {
      if ( preg_match("/_enable$/", $k) && $v == 1 ) {
        $a = explode('_', $k);
        $key = $a[0];
        $dd[$key] = $this->default_geocoder_services[$key]['label'];
      }
    }
    add_settings_field(
      'geocoder_service',                            // ID used to identify the field throughout the theme
      __( 'Geocoder service', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'select_element_callback'),      // The name of the function responsible for rendering the option interface
      $whichpage,                                    // The page on which this option will be displayed
      $whichsection,                                 // The name of the section to which this field belongs
      array(                                         // The array of arguments to pass to the callback.
        'id' => 'geocoder_service',
        'select_opts' => $dd,
        'default_opt' => $this->default_leaflet_plugin_settings['geocoder']['service'],
        'class' => 'leaflet_geocoder',
        'opt' => $saveto
      )
    );

    add_settings_field(
      'geocoder_position',                                           // ID used to identify the field throughout the theme
      __( 'Position of the Geocoder button', 'getlocations-lite' ),  // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                      // The name of the function responsible for rendering the option interface
      $whichpage,                                                    // The page on which this option will be displayed
      $whichsection,                                                 // The name of the section to which this field belongs
      array(                                                         // The array of arguments to pass to the callback.
        'id' => 'geocoder_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['geocoder']['position'],
        'class' => 'leaflet_geocoder',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'geocoder_placeholder',                   // ID used to identify the field throughout the theme
      __( 'Placeholder', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'text_element_callback'),   // The name of the function responsible for rendering the option interface
      $whichpage,                               // The page on which this option will be displayed
      $whichsection,                            // The name of the section to which this field belongs
      array(                                    // The array of arguments to pass to the callback.
        'id' => 'geocoder_placeholder',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_geocoder',
        'default_opt' => $this->default_leaflet_plugin_settings['geocoder']['placeholder'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'geocoder_label',                        // ID used to identify the field throughout the theme
      __( 'Label', 'getlocations-lite' ),      // The label to the left of the option interface element
      array( $this, 'text_element_callback'),  // The name of the function responsible for rendering the option interface
      $whichpage,                              // The page on which this option will be displayed
      $whichsection,                           // The name of the section to which this field belongs
      array(                                   // The array of arguments to pass to the callback.
        'id' => 'geocoder_label',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_geocoder',
        'default_opt' => $this->default_leaflet_plugin_settings['geocoder']['label'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'geocoder_click',                                      // ID used to identify the field throughout the theme
      __( 'Reverse Geocode on click', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),            // The name of the function responsible for rendering the option interface
      $whichpage,                                            // The page on which this option will be displayed
      $whichsection,                                         // The name of the section to which this field belongs
      array(                                                 // The array of arguments to pass to the callback.
        'id' => 'geocoder_click',
        'desc' => __( 'Click on map to reverse geocode.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['geocoder']['click'],
        'class' => 'leaflet_geocoder',
        'opt' => $saveto
      )
    );
    if ( $share->get_countries_list() ) {
      $countries = $share->get_countries_list(__('No limit', 'getlocations-lite'));
      add_settings_field(
        'geocoder_country',                                     // ID used to identify the field throughout the theme
        __( 'Limit search to a country', 'getlocations-lite' ), // The label to the left of the option interface element
        array( $this, 'select_element_callback'),               // The name of the function responsible for rendering the option interface
        $whichpage,                                             // The page on which this option will be displayed
        $whichsection,                                          // The name of the section to which this field belongs
        array(                                                  // The array of arguments to pass to the callback.
          'id' => 'geocoder_country',
          'desc' => __( 'Where available.', 'getlocations-lite' ),
          'select_opts' => $countries,
          'default_opt' => $this->default_leaflet_plugin_settings['geocoder']['country'],
          'class' => 'leaflet_geocoder',
          'opt' => $saveto
        )
      );
    }
    // geocoder services?
  }
  // leaflet_plugin_kml
  if ( isset($plugins['leaflet_plugin_kml']) && $plugins['leaflet_plugin_kml'] ) {
    add_settings_field(
      'kml_enable',                               // ID used to identify the field throughout the theme
      __( 'Enable KML', 'getlocations-lite' ),    // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'), // The name of the function responsible for rendering the option interface
      $whichpage,                                 // The page on which this option will be displayed
      $whichsection,                              // The name of the section to which this field belongs
      array(                                      // The array of arguments to pass to the callback.
        'id' => 'kml_enable',
        'desc' => __( 'Enable KML layer on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['kml']['enable'],
        'class' => 'settings_section_top',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'kml_url',                                    // ID used to identify the field throughout the theme
      __( 'KML File', 'getlocations-lite' ),        // The label to the left of the option interface element
      array( $this, 'kml_media_selector_callback'), // The name of the function responsible for rendering the option interface
      $whichpage,                                   // The page on which this option will be displayed
      $whichsection,                                // The name of the section to which this field belongs
      array(                                        // The array of arguments to pass to the callback.
        'id' => 'kml_url',
        'class' => 'leaflet_kml',
        'default_opt' => $this->default_leaflet_plugin_settings['kml']['url'],
        #'desc' => __( 'Select a KML file from the Media library.', 'getlocations-lite' ),
        'opt' => $saveto
      )
    );
    add_settings_field(
      'kml_enable_button',                            // ID used to identify the field throughout the theme
      __( 'Enable KML button', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),     // The name of the function responsible for rendering the option interface
      $whichpage,                                     // The page on which this option will be displayed
      $whichsection,                                  // The name of the section to which this field belongs
      array(                                          // The array of arguments to pass to the callback.
        'id' => 'kml_enable_button',
        'desc' => __( 'Enable KML button on the map.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['kml']['enable_button'],
        'class' => 'leaflet_kml',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'kml_position',                                          // ID used to identify the field throughout the theme
      __( 'Position of the KML button', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'select_element_callback'),                // The name of the function responsible for rendering the option interface
      $whichpage,                                              // The page on which this option will be displayed
      $whichsection,                                           // The name of the section to which this field belongs
      array(                                                   // The array of arguments to pass to the callback.
        'id' => 'kml_position',
        'select_opts' => $this->controlpositions,
        'default_opt' => $this->default_leaflet_plugin_settings['kml']['position'],
        'class' => 'leaflet_kml',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'kml_label',                            // ID used to identify the field throughout the theme
      __( 'Label', 'getlocations-lite' ),     // The label to the left of the option interface element
      array( $this, 'text_element_callback'), // The name of the function responsible for rendering the option interface
      $whichpage,                             // The page on which this option will be displayed
      $whichsection,                          // The name of the section to which this field belongs
      array(                                  // The array of arguments to pass to the callback.
        'id' => 'kml_label',
        'widthclass' => 'medium-text',
        'class' => 'leaflet_kml',
        'default_opt' => $this->default_leaflet_plugin_settings['kml']['label'],
        'opt' => $saveto
      )
    );
    add_settings_field(
      'kml_initial_state',                            // ID used to identify the field throughout the theme
      __( 'KML Initial State', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),     // The name of the function responsible for rendering the option interface
      $whichpage,                                     // The page on which this option will be displayed
      $whichsection,                                  // The name of the section to which this field belongs
      array(                                          // The array of arguments to pass to the callback.
        'id' => 'kml_initial_state',
        'desc' => __( 'Initial state, on/off of KLM layer.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['kml']['initial_state'],
        'class' => 'leaflet_kml',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'kml_viewport',                                 // ID used to identify the field throughout the theme
      __( 'Preserve viewport', 'getlocations-lite' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),     // The name of the function responsible for rendering the option interface
      $whichpage,                                     // The page on which this option will be displayed
      $whichsection,                                  // The name of the section to which this field belongs
      array(                                          // The array of arguments to pass to the callback.
        'id' => 'kml_viewport',
        'desc' => __( 'Preserve viewport of KLM layer.', 'getlocations-lite' ),
        'default_opt' => $this->default_leaflet_plugin_settings['kml']['viewport'],
        'class' => 'leaflet_kml',
        'opt' => $saveto
      )
    );
    add_settings_field(
      'kml_forceseparatebutton',                        // ID used to identify the field throughout the theme
      __( 'Keep button separate', 'getlocations-pro' ), // The label to the left of the option interface element
      array( $this, 'checkbox_element_callback'),       // The name of the function responsible for rendering the option interface
      $whichpage,                                       // The page on which this option will be displayed
      $whichsection,                                    // The name of the section to which this field belongs
      array(                                            // The array of arguments to pass to the callback.
        'id' => 'kml_forceseparatebutton',
        'desc' => __( 'Keep button separate from zoom button.', 'getlocations-pro' ),
        'default_opt' => $this->default_leaflet_plugin_settings['kml']['forceseparatebutton'],
        'class' => 'leaflet_kml',
        'opt' => $saveto
      )
    );
  }
  // leaflet_plugin_awesome
  $foundawesome = $share->check_fontawesome_install();
  if ( isset($plugins['leaflet_plugin_awesome']) && $plugins['leaflet_plugin_awesome'] ) {
    if ( $foundawesome ) {
      add_settings_field(
        'awesome_enable',                                        // ID used to identify the field throughout the theme
        __( 'Enable Fontawesome markers', 'getlocations-lite' ), // The label to the left of the option interface element
        array( $this, 'checkbox_element_callback'),              // The name of the function responsible for rendering the option interface
        $whichpage,                                              // The page on which this option will be displayed
        $whichsection,                                           // The name of the section to which this field belongs
        array(                                                   // The array of arguments to pass to the callback.
          'id' => 'awesome_enable',
          'desc' => __( 'Enable Fontawesome markers on the map.', 'getlocations-lite' ),
          'default_opt' => $this->default_leaflet_plugin_settings['awesome']['enable'],
          'class' => 'settings_section_top',
          'opt' => $saveto
        )
      );
    }
  }
