<?php
/**
 * @file    getlocations-lite.php
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 *
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @wordpress-plugin
 * Plugin Name:       Getlocations Lite
 * Plugin URI:        https://www.arwystli.co.uk/getlocations-lite
 * Description:       Getlocations Lite provides maps using the Leaflet software.
 * Version:           1.0.5
 * Author:            Bob Hutchinson
 * Author URI:        arwystli.co.uk/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       getlocations-lite
 * Domain Path:       /languages
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
  die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */

// get the wordpress-plugin data so we can pull out the version from the above headers
if ( is_admin() ) {
  if( ! function_exists('get_plugin_data') ){
    require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
  }
  $plugin_data = get_plugin_data( __FILE__ );
  define( 'GETLOCATIONS_LITE_VERSION', $plugin_data['Version'] );
}

define( 'GETLOCATIONS_LITE_DB_VERSION', '1.0.0');
define( 'GETLOCATIONS_LITE_PLUGIN_FILE', __FILE__ );
define( 'GETLOCATIONS_LITE_PLUGIN_DIR', plugin_dir_path(__FILE__) );
define( 'GETLOCATIONS_LITE_PLUGIN_ADMIN_TEMPLATES_DIR', GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/templates/' );
define( 'GETLOCATIONS_LITE_PLUGIN_PUBLIC_TEMPLATES_DIR', GETLOCATIONS_LITE_PLUGIN_DIR . 'public/templates/' );
define( 'GETLOCATIONS_LITE_PLUGIN_DIR_URL', plugin_dir_url(__FILE__) );
define( 'GETLOCATIONS_LITE_PLUGIN_SLUG', 'getlocations-lite' );
define( 'GETLOCATIONS_LITE_IONICONS_FILE', GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/js/ionicons-2.0.1.json' );
define( 'GETLOCATIONS_LITE_DASHICONS_FILE', GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/js/dashicons.json' );
define( 'GETLOCATIONS_LITE_AWESOME_FILE', GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/js/fawesome-5.12.1.json');
if ( ! defined('GETLOCATIONS_COMMON')) {
  define( 'GETLOCATIONS_COMMON', 'getlocations' );
}

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-getlocations-lite-activator.php
 */
function activate_getlocations_lite() {
  require_once plugin_dir_path( __FILE__ ) . 'includes/class-getlocations-lite-activator.php';
  Getlocations_Lite_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-getlocations-lite-deactivator.php
 */
function deactivate_getlocations_lite() {
  require_once plugin_dir_path( __FILE__ ) . 'includes/class-getlocations-lite-deactivator.php';
  Getlocations_Lite_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_getlocations_lite' );
register_deactivation_hook( __FILE__, 'deactivate_getlocations_lite' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-getlocations-lite.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_getlocations_lite() {

  $plugin = new Getlocations_Lite();
  $plugin->run();

}
run_getlocations_lite();
