<?php
/**
 * @file    class-getlocations-lite-share.php
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 *
 * A class shared between admin and public
 */

class Getlocations_Lite_Share {

  public function __construct() {

    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/common_vars.php');
    $this->default_leaflet_plugin_settings = $default_leaflet_plugin_settings;
    $this->default_leaflet_plugin_keys = $default_leaflet_plugin_keys;
    $this->plugins_bool = $plugins_bool;
    $this->default_settings = $default_settings;
    $this->default_settings_mapped = $default_settings_mapped;
    $this->bool = $bool;
    $this->default_geocoder_services = $default_geocoder_services;

    $mp  = get_option('getlocations_lite_providers');
    $map_key_data = get_option('getlocations_lite_map_keys');
    // ensure that the api key is brought in to map providers
    if ( isset($map_key_data['Thunderforest']) ) {
      $thunderforest_key = $map_key_data['Thunderforest'];
    }
    if ( isset($map_key_data['Openweathermap']) ) {
      $openweathermap_key = $map_key_data['Openweathermap'];
    }
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/providers.php');
    foreach ( $mp AS $k => $v  ) {
      if ( isset($default_leaflet_providers_data[$k]) && $v == '1' ) {
        $mp[$k] = $default_leaflet_providers_data[$k];
      }
    }
    $this->map_providers = $mp;

  }

  public function testit() {
    return 'TESTIT NOW';
  }

  /**
   * $markers contains the required markers in an array
   * @param $markers array or empty string
   * @return string
   */
  public function get_marker_script($markers = '', $id = 1) {
    if ( is_array($markers) ) {
      $info = $this->get_marker_data2($markers);
    }
    else {
      $info = $this->get_all_marker_data();
    }

    $contents = "// Getlocations_lite marker image data.\n";
    if ($id < 2) {
      $contents .= "var getlocations_lite_iconinfo = [];\n";
    }
    $contents .= 'getlocations_lite_iconinfo["key_' . $id . '"] = ' . json_encode($info) . ";\n";
    return $contents;
  }

  // latlon_check
  public function latlon_check($latlon) {

    if (! empty($latlon)) {
      $ll = explode(',', $latlon);
      $lat = (isset( $ll[0]) ? $ll[0] : FALSE);
      $lon = (isset( $ll[1]) ? $ll[1] : FALSE);
      $lat = trim($lat);
      $lon = trim($lon);

      if ((abs($lat) || abs($lon)) && is_numeric($lat) && is_numeric($lon)) {
        $latregex = "/[-+]{0,1}[0-9]{1,}\.{1}[0-9]{1,}/";
        if (! strpos($lat, '.')) {
          $latregex = "/[-+]{0,1}[0-9]{1,}/";
        }
        $lonregex = "/[-+]{0,1}[0-9]{1,}\.{1}[0-9]{1,}/";
        if (! strpos($lon, '.')) {
          $lonregex = "/[-+]{0,1}[0-9]{1,}/";
        }
        if (preg_match($latregex, $lat) && preg_match($lonregex, $lon)) {
          // Normalize coordinates.
          $lat = $this->normalizelat($lat);
          $lon = $this->normalizelng($lon);
          return $lat . "," . $lon;
        }
      }
    }
    return FALSE;
  }

  /**
  * Normalizes a latitude to the [-90,90] range. Latitudes above 90 or
  * below -90 are capped, not wrapped.
  * @param {Number} lat The latitude to normalize, in degrees.
  * @type Number
  * @return Returns the latitude, fit within the [-90,90] range.
  */
  public function normalizelat($lat) {
    return max(-90, min(90, $lat));
  }

  /**
  * Normalizes a longitude to the [-180,180] range. Longitudes above 180
  * or below -180 are wrapped.
  * @param {Number} lng The longitude to normalize, in degrees.
  * @type Number
  * @return Returns the longitude, fit within the [-180,180] range.
  */
  public function normalizelng($lng) {
    if ($lng % 360 == 180) {
        return 180;
      }
      while ($lng > 180) {
        $lng -= 360;
      }
      while ($lng < -180) {
        $lng += 360;
      }
      return $lng;
  }


  /**
  * @param int $ct $location $minmaxes
  *
  * @return
  *   Returns $minmaxes
  *
  */
  public function do_minmaxes($ct, $location, $minmaxes) {
    if ($minmaxes == '') {
      $minmaxes = array('minlat' => 0, 'minlon' => 0, 'maxlat' => 0, 'maxlon' => 0);
    }
    if ($ct) {
      // latitudes
      if ($location['latitude'] > $minmaxes['maxlat']) {
        $minmaxes['maxlat'] = $location['latitude'];
      }
      if ($location['latitude'] < $minmaxes['minlat']) {
        $minmaxes['minlat'] = $location['latitude'];
      }
      // longitudes
      if ($location['longitude'] > $minmaxes['maxlon']) {
        $minmaxes['maxlon'] = $location['longitude'];
      }
      if ($location['longitude'] < $minmaxes['minlon']) {
        $minmaxes['minlon'] = $location['longitude'];
      }
    }
    else {
      $minmaxes['minlat'] = $location['latitude'];
      $minmaxes['minlon'] = $location['longitude'];
      $minmaxes['maxlat'] = $location['latitude'];
      $minmaxes['maxlon'] = $location['longitude'];
    }
    return $minmaxes;
  }

  /**
   * Adapted from Drupal 7
   *
   * Finds all files that match a given mask in a given directory.
   *
   * Directories and files beginning with a period are excluded; this
   * prevents hidden files and directories (such as SVN working directories)
   * from being scanned.
   *
   * @param $dir
   *   The base directory or URI to scan, without trailing slash.
   * @param $mask
   *   The preg_match() regular expression of the files to find.
   * @param $options
   *   An associative array of additional options, with the following elements:
   *   - 'nomask': The preg_match() regular expression of the files to ignore.
   *     Defaults to '/(\.\.?|CVS)$/'.
   *   - 'callback': The callback function to call for each match. There is no
   *     default callback.
   *   - 'recurse': When TRUE, the directory scan will recurse the entire tree
   *     starting at the provided directory. Defaults to TRUE.
   *   - 'key': The key to be used for the returned associative array of files.
   *     Possible values are 'uri', for the file's URI; 'filename', for the
   *     basename of the file; and 'name' for the name of the file without the
   *     extension. Defaults to 'uri'.
   *   - 'min_depth': Minimum depth of directories to return files from. Defaults
   *     to 0.
   * @param $depth
   *   Current depth of recursion. This parameter is only used internally and
   *   should not be passed in.
   *
   * @return
   *   An associative array (keyed on the chosen key) of objects with 'uri',
   *   'filename', and 'name' members corresponding to the matching files.
   */
  public function file_scan_directory($dir, $mask, $options = array(), $depth = 0) {
    // Default nomask option.
    $nomask = '/(\.\.?|CVS)$/';

    // Merge in defaults.
    $options += array(
      'nomask' => $nomask,
      'callback' => 0,
      'recurse' => TRUE,
      'key' => 'uri',
      'min_depth' => 0,
    );

    $options['key'] = in_array($options['key'], array('uri', 'filename', 'name'), TRUE ) ? $options['key'] : 'uri';
    $files = array();
    if (is_dir($dir) && $handle = opendir($dir)) {
      while (FALSE !== ($filename = readdir($handle))) {
        if (!preg_match($options['nomask'], $filename) && $filename[0] != '.') {
          $uri = "$dir/$filename";
          if (is_dir($uri) && $options['recurse']) {
            $files = array_merge($this->file_scan_directory($uri, $mask, $options, $depth + 1), $files);
          }
          elseif ($depth >= $options['min_depth'] && preg_match($mask, $filename)) {
            // Always use this match over anything already set in $files with the
            // same $$options['key'].
            $file = new stdClass();
            $file->uri = $uri;
            $file->filename = $filename;
            $file->name = pathinfo($filename, PATHINFO_FILENAME);
            $key = $options['key'];
            $files[$file->$key] = $file;
            if ($options['callback']) {
              $options['callback']($uri);
            }
          }
        }
      }
      closedir($handle);
    }
    return $files;
  }

  /**
   * Build a data array for markers
   * @param $markerdir Where the markers are, eg public/markers
   * @param $markerurl Where the markers can be downloaded
   * @param $reset if true rebuild the data
   * @return array
   */
  public function make_marker_data($markerdir, $markerurl, $reset = FALSE) {

    // do we already have the data
    $opts = get_option('getlocations_lite_marker_data');
    if ( ! $reset && $opts ) {
      return $opts;
    }


    // each markerdir contains a json file with data about the set
    // named eg colors.json
    if ( file_exists($markerdir) && is_dir($markerdir) ) {

      // get json files first, one for each group
      $jsondata = array();
      $reg = '/.*\.json$/';
      $reg2 = '/.*\.png$/';
      $markerdir_json_scan = $this->file_scan_directory($markerdir, $reg);

      // loop over jsonfiles
      $data = array();
      foreach ( $markerdir_json_scan AS $key => $obj ) {
        $group = $obj->name;

        if ( file_exists($key) ) {
          $c_js = file_get_contents($key);
          $data[$group] = json_decode($c_js, TRUE);
          // get icons info per group
          $groupdir = $markerdir . DIRECTORY_SEPARATOR . $group;
          $markerdir_scan = $this->file_scan_directory($groupdir, $reg2);
          foreach ( $markerdir_scan AS $path => $obj2 ) {
            if ($obj2->name !== 'shadow') {
              $filename = $obj2->uri;
              $datakey = $group . ' ' . $obj2->name;
              $position = '';
              if ( isset($data[$group][$datakey]['position']) ) {
                $position = $data[$group][$datakey]['position'];
              }
            }
            if ( file_exists($markerdir  . DIRECTORY_SEPARATOR . $group . DIRECTORY_SEPARATOR . 'shadow.png') ) {
              $shadowurl = $markerurl . '/' . $group . '/' . 'shadow.png';
              list($width, $height, $type, $attr) = getimagesize($markerdir  . DIRECTORY_SEPARATOR . $group . DIRECTORY_SEPARATOR . 'shadow.png');
              $shadowsize = $width . "," . $height;
              $shadowanchor =  intval( ($width / 2) - ($width / 4) ) . "," . $height;
            }
            else {
              $shadowurl = '';
              $shadowsize = '';
              $shadowanchor = '';
            }
            // skip shadow
            if ($obj2->name !== 'shadow') {
              if ( file_exists($filename) ) {
                list($width, $height, $type, $attr) = getimagesize($filename);
                $iconsize = $width . ',' . $height;
                if ($position == 'bottom') {
                  $iconanchor = intval($width / 2) . ',' . $height;
                  $popupanchor = "0," . intval(-$height);
                }
                elseif  ($position == 'center') {
                  $iconanchor = intval($width / 2) . ',' . intval($height / 2);
                  $popupanchor = "0," . "-" . intval($height/2);
                }
              }

              $data[$group][$datakey]['iconinfo']['iconUrl']       = $markerurl . '/' . $group . '/' . $obj2->filename;
              $data[$group][$datakey]['iconinfo']['iconSize']      = $iconsize;
              $data[$group][$datakey]['iconinfo']['iconAnchor']    = $iconanchor;
              $data[$group][$datakey]['iconinfo']['popupAnchor']   = $popupanchor;
              $data[$group][$datakey]['iconinfo']['tooltipAnchor'] = $popupanchor;
              if ( $shadowurl ) {
                $data[$group][$datakey]['iconinfo']['shadowUrl']    = $shadowurl;
                $data[$group][$datakey]['iconinfo']['shadowSize']   = $shadowsize;
                $data[$group][$datakey]['iconinfo']['shadowAnchor'] = $shadowanchor;
              }
            }
          }
        }
        else {
          echo 'File NOT found: ' .$key . '<br>';
        }
      }

      if ($reset) {
        delete_option('getlocations_lite_marker_data');
      }
      $opts = get_option('getlocations_lite_marker_data');
      if (! $opts ) {
        add_option('getlocations_lite_marker_data', $data);
        $opts = get_option('getlocations_lite_marker_data');
      }
      return $opts;
    }
    return FALSE;
  }

  /**
   * Make a dropdown of markers
   *
   * @return string
   */
  public function get_marker_data_dd($sel_marker = '', $name = 'icon' ) {
    $data = get_option('getlocations_lite_marker_data');
    $output = '<select id="' . $name . '" name="' . $name . '" >';
    foreach ( $data AS $k => $v ) {
      foreach ( $v AS $k2 => $v2 ) {
        $sel = '';
        if ( $k2 == $sel_marker ) {
          $sel = ' selected="selected"';
        }
        $output .= '<option value="' . $k2 . '"' . $sel . '>' . $v2['label'] . '</option>';
      }
    }
    $output .= '</select>';
    return $output;
  }

  /**
   * Get icon info for an array of icons, by key eg 'colors green'
   *
   *
   */
  public function get_marker_data($markers ) {
    if ( empty($markers) ) {
      return FALSE;
    }
    if ( is_string($markers) ) {
      $markers = array($markers);
    }
    $data = get_option('getlocations_lite_marker_data');
    $info = array();
    foreach ( $data AS $k => $v ) {
      foreach ( $v AS $k2 => $v2 ) {
        if ( in_array( $k2, $markers, TRUE ) ) {
          $info[$k2] = $v2['iconinfo'];
        }
      }
    }
    return $info;
  }

  /**
   * should return only the icons for a particular map or set of markers
   * @param $markers array
   * @return array $info or FALSE
   */
  public function get_marker_data2($markers ) {
    if ( empty($markers) || ! is_array($markers) ) {
      return FALSE;
    }

    // $markers we need keyed on 'icon' which contains the icon keys, eg 'colors green'
    $marr = array();
    foreach ($markers AS $k => $v) {
      $marr[] = $markers[$k]['icon'];
    }

    $data = get_option('getlocations_lite_marker_data');
    $info = array();
    foreach ( $data AS $k => $v ) {
      foreach ( $v AS $k2 => $v2 ) {
        if ( in_array( $k2, $marr, TRUE ) ) {
          $info[$k2] = $v2['iconinfo'];
        }
      }
    }
    return $info;
  }

  /**
   * Provide an array of marker names keyed on thei ids, eg 'colors green' suitable for dd
   *
   * @return array
   */
  public function get_marker_titles() {
    $data = get_option('getlocations_lite_marker_data');
    $out = array();
    foreach ( $data AS $k => $v ) {
      foreach ( $v AS $k2 => $v2 ) {
        $out[$k2] = $v2['label'];
      }
    }
    return $out;
  }

  /**
   * Provide all the marker data iconinfo
   *
   * @return array
   */
  public function get_all_marker_data() {
    $data = get_option('getlocations_lite_marker_data');
    $info = array();
    foreach ( $data AS $k => $v ) {
      foreach ( $v AS $k2 => $v2 ) {
        $info[$k2] = $v2['iconinfo'];
      }
    }
    return $info;
  }

  /**
   * @return array all marker groups
   *
   */
#  public function get_marker_groups() {
#    $data = get_option('getlocations_lite_marker_data');
#    $groups = array();
#    foreach ( $data AS $k => $v ) {
#      $groups[] = $k;
#    }
#    return $groups;
#  }


#######################################################################
  /**
   * Build $map_plugins_opts for inclusion in the DOM
   * 1. $default_leaflet_plugin_settings
   * 2. override with saved settings: get_option('getlocations_lite_plugins_settings')
   * 3. override with $in which might come from a saved map or shortcode
   * 4. available plugins from get_option('getlocations_lite_plugins')
   * 5. will need to be converted from old_k to new_k
   *
   * @param array $in
   * @return array
   */
  public function build_map_plugins_opts($in = FALSE) {

    $ps1 = $this->default_leaflet_plugin_settings; // in new_k format
    $ps2 = get_option('getlocations_lite_plugins_settings', array()); // in old_k format

    // get available_plugins
    $available_plugins = $this->get_available_plugins();

    // get lowercase bool
    $lbool = array();
    foreach ( $this->bool AS $tb ) {
      $lbool[] = strtolower($tb);
    }

    if ( count($ps2) ) {
      // convert $ps2
      $ps2n = array();
      foreach ( $this->default_leaflet_plugin_keys AS $pl => $ks ) {
        // only available_plugins
        if ( in_array($pl, $available_plugins, TRUE ) ) {
          foreach ( $ks AS $oldk => $newk ) {
            if ( isset( $ps2[$oldk] ) ) {
              $ps2n[$pl][$newk] = $ps2[$oldk];
            }
            else {
              if ( in_array($newk, $lbool, TRUE ) ) {
                $ps2n[$pl][$newk] = 0;
              }
            }

          }
        }
      }
      if ( count($ps2n) ) {
        $ps2 = $ps2n;
      }

      // now $ps1 and $ps2 are in the same format use array_replace_recursive() merge
      $ps3 = array_replace_recursive($ps1, $ps2);

    }
    else {
      $ps3 = $ps1;
    }

    $ps4 = array();
    // look at $in
    // anything there?
    if ( $in && is_array($in) && count($in) ) {
      // need to test wether it needs converting
      $found = FALSE;
      foreach ( $in AS $pl => $ks ) {
        if ( in_array($pl, $available_plugins, TRUE ) ) {
          $found = TRUE;
        }
      }
      if ( ! $found ) {
        // needs converting
        $inn = array();
        foreach ( $this->default_leaflet_plugin_keys AS $pl => $ks ) {
          // only available_plugins
          if ( in_array($pl, $available_plugins, TRUE ) ) {
            foreach ( $ks AS $oldk => $newk ) {
              if ( isset( $in[$oldk] ) ) {
                $inn[$pl][$newk] = $in[$oldk];
              }
            }

          }
        }
        if ( count($inn) ) {
          $in = $inn;
        }
      }
      $ps4 = array_replace_recursive($ps3, $in);
    }
    if ( count($ps4) ) {
      $mpo = $ps4;
    }
    else {
      $mpo = $ps3;
    }

    // now make bool and eliminate not available_plugins
    foreach ( $mpo AS  $pl => $ks ) {
      if ( in_array($pl, $available_plugins, TRUE ) ) {
        foreach ( $ks AS $k => $v ) {
          if ( in_array( $k, $this->plugins_bool[$pl], TRUE ) ) {
            if ( $v == 1 || $v == '1' ) {
              $mpo[$pl][$k] = TRUE;
            }
            else {
              $mpo[$pl][$k] = FALSE;
            }
          }
        }
      }
      else {
        // delete unavailable plugins
        unset( $mpo[$pl] );
      }
    }

    // some $mpo addins
    if ( isset( $mpo['graticule'] ) ) {
      if ( file_exists(GETLOCATIONS_LITE_PLUGIN_DIR . 'public/js/plugins/leaflet.latlng-graticule/intervals.php') ) {
        include(GETLOCATIONS_LITE_PLUGIN_DIR . 'public/js/plugins/leaflet.latlng-graticule/intervals.php');
        $mpo['graticule']['zoomInterval']['latitude'] = $horizontal_intervals;
        $mpo['graticule']['zoomInterval']['longitude'] = $vertical_intervals;
      }
    }
    if ( isset( $mpo['markercluster'] ) ) {
      if ( file_exists(GETLOCATIONS_LITE_PLUGIN_DIR . 'public/js/plugins/Leaflet.markercluster/markerclusteroptions.php') ) {
        include(GETLOCATIONS_LITE_PLUGIN_DIR . 'public/js/plugins/Leaflet.markercluster/markerclusteroptions.php');
        $mpo['markercluster']['markerclusteroptions'] = $markerclusteroptions;
      }
    }
    if ( isset( $mpo['geocoder'] ) ) {
      $geocoders = get_option('getlocations_lite_geocoders');
      $s = $mpo['geocoder']['service'];
      $mpo['geocoder']['settings']['needkey'] = $this->default_geocoder_services[$s]['key'];
      $mpo['geocoder']['settings']['countrycodes'] = $this->default_geocoder_services[$s]['countrycodes'];
      $mpo['geocoder']['settings']['reverse'] = $this->default_geocoder_services[$s]['reverse'];
      $mpo['geocoder']['settings']['label'] = $this->default_geocoder_services[$s]['label'];
      #$mpo['geocoder']['label'] = $mpo['geocoder']['label'] . __(' by ') . $this->default_geocoder_services[$s]['label'];
      if ( isset( $geocoders[$s . '_enable'] ) && $geocoders[$s . '_enable'] == 1 ) {
        if ( isset( $geocoders[$s . '_key'] ) && ! empty($geocoders[$s . '_key']) ) {
          $mpo['geocoder']['settings']['key'] = $geocoders[$s . '_key'];
        }
        // neutrino
        if ( isset( $geocoders[$s . '_userId'] ) && ! empty($geocoders[$s . '_userId']) && isset( $geocoders[$s . '_apiKey'] ) && ! empty($geocoders[$s . '_apiKey']) ) {
          $mpo['geocoder']['settings']['userId'] = $geocoders[$s . '_userId'];
          $mpo['geocoder']['settings']['apiKey'] = $geocoders[$s . '_apiKey'];
        }
      }
    }
    return $mpo;
  }

  /**
   * Build $map_plugins_opts for inclusion in the DOM
   * 1. $this->default_settings from common_vars
   * 2. override with saved settings: get_option('getlocations_lite_default_settings')
   * 3. override with $in which might come from a saved map or shortcode
   * 4. $this->default_settings_mapped
   *
   * @param array $in
   * @return array
   */
  public function build_map_opts($in = FALSE) {
    $ds1 = $this->default_settings; // lowercase keys
    $ds2 = get_option('getlocations_lite_default_settings'); // lowercase keys

    $lbool = array();
    foreach ( $this->bool AS $tb ) {
      $lbool[] = strtolower($tb);
    }
    foreach ( array_keys($ds1) AS $k ) {
      if ( ! isset($ds2[$k]) && in_array($k, $lbool, TRUE ) ) {
        $ds2[$k] = 0;
      }
    }

    $ds3 = array_replace_recursive($ds1, $ds2);

    $ds4 = array();
    // look at $in
    // anything there?
    if ( $in && is_array($in) && count($in) ) {
      $ds4 = array_replace_recursive($ds3, $in);
    }
    else {
      $ds4 = $ds3;
    }
    $mo = $ds4;

    $default_map_provider = $this->default_settings['map_provider'];
    if ( isset($mo['map_provider']) && $mo['map_provider'] ) {
      $default_map_provider = $mo['map_provider'];
    }
    else {
      $mo['map_provider'] = $default_map_provider;
    }

    // do bool
    foreach ($mo AS $k => $v) {
      if (in_array($k, $this->bool, TRUE ) ) {
        if ($v == "1" || $v == 1 ) {
          $mo[$k] = TRUE;
        }
        else {
          $mo[$k] = FALSE;
        }
      }
    }

    return $mo;
  }

  /**
   * @param array $mo
   * Convert keys to camelcase and set boolean vars to TRUE/FALSE ready for js
   * @return array $mo2
   *
   */
  public function get_map_opts_mapped($mo) {

    $mo2 = array();
    $newkey = '';
    foreach( $mo AS $optkey => $optval ) {
      if ( isset( $this->default_settings_mapped[$optkey] ) ) {
        $newkey = $this->default_settings_mapped[$optkey];
      }
      else {
        $newkey = $optkey;
      }
      $mo2[ $newkey ] = $mo[ $optkey ];
      // do bool to newkey values
      if (in_array($newkey, $this->bool, TRUE ) ) {
        if ( $mo2[ $newkey ] == '1' || $mo2[ $newkey ] == '1' ) {
          $mo2[ $newkey ] = TRUE;
        }
        else {
          $mo2[ $newkey ] = FALSE;
        }
      }
    }
    return $mo2;

  }

  /**
   * @param array
   *
   * converts plugins key from old to new format:
   * array('graticule_enable => 1) becomes array('graticule => array('enable' => 1))
   *
   */
  public function convert_plugins_keys($in = FALSE) {

    $available_plugins = $this->get_available_plugins();

    $out = array();
    if ( $in && is_array($in) && count($in) ) {
      // need to test wether it needs converting
      $found = FALSE;
      foreach ( $in AS $pl => $ks ) {
        if ( in_array($pl, $available_plugins, TRUE ) ) {
          $found = TRUE;
        }
      }
      if ( ! $found ) {
        // needs converting
        foreach ( $this->default_leaflet_plugin_keys AS $pl => $ks ) {
          // only available_plugins
          if ( in_array($pl, $available_plugins, TRUE ) ) {
            foreach ( $ks AS $oldk => $newk ) {
              if ( isset( $in[$oldk] ) ) {
                $out[$pl][$newk] = $in[$oldk];
                if ( in_array($newk, $this->plugins_bool[$pl], TRUE ) ) {
                  if ( $in[$oldk] == 1 || $in[$oldk] == '1' ) {
                    $out[$pl][$newk] = TRUE;
                  }
                  else {
                    $out[$pl][$newk] = FALSE;
                  }
                }
              }
            }
          }
        }
      }
    }
    return $out;
  }

  /**
   * Provides an array of enabled leaflet plugins
   *
   *
   * @return array
   */
  public function get_available_plugins() {
    $ap = get_option('getlocations_lite_plugins');
    $available_plugins = array();
    if ( is_array($ap) && count($ap) ) {
      foreach ( $ap AS $k => $v ) {
        // only if activated
        if ( $v ) {
          $k2 = preg_replace("/leaflet_plugin_/", '', $k);
          $available_plugins[] = $k2;
        }
      }
    }
    return $available_plugins;
  }

  /**
  * ionicons data
  *
  *
  */
  public function get_ionicons_data( $return_as_js = TRUE ) {
    if ( file_exists(GETLOCATIONS_LITE_IONICONS_FILE) ) {
      $data = file_get_contents(GETLOCATIONS_LITE_IONICONS_FILE);
      if ( $return_as_js ) {
        return $data;
      }
      return json_decode($data, TRUE);
    }
    return FALSE;
  }

  /**
  * dashicons data
  *
  *
  */
  public function get_dashicons_data( $return_as_js = TRUE ) {
    if ( file_exists(GETLOCATIONS_LITE_DASHICONS_FILE) ) {
      $data = file_get_contents(GETLOCATIONS_LITE_DASHICONS_FILE);
      if ( $return_as_js ) {
        return $data;
      }
      return json_decode($data, TRUE);
    }
    return FALSE;
  }

  /**
  * awesome data
  *
  *
  */
  public function get_awesome_data( $return_as_js = TRUE ) {
    if ( file_exists(GETLOCATIONS_LITE_AWESOME_FILE) ) {
      $data = file_get_contents(GETLOCATIONS_LITE_AWESOME_FILE);
      if ( $return_as_js ) {
        return $data;
      }
      return json_decode($data, TRUE);
    }
    return FALSE;
  }

  public function check_fontawesome_install() {
    $apl = get_option('active_plugins');
    $foundawesome = FALSE;
    foreach ( $apl AS $k => $v ) {
      if ( $v == "font-awesome/font-awesome.php" || $v == "font-awesome/index.php" ) {
        $foundawesome = TRUE;
      }
    }
    return $foundawesome;
  }

  // countries
  public function get_countries_list($blank = '') {
    include_once(GETLOCATIONS_LITE_PLUGIN_DIR . 'includes/class-getlocations-lite-countries.php');
    $countries = new Getlocations_Lite_Countries();
    return $countries->countries_list($blank);
  }

  // more here


}

