<?php
/**
 * @file    class-getlocations-lite-public.php
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 *
 * The public-facing functionality of the plugin.
 */

class Getlocations_Lite_Public {

  /**
   * The ID of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param string  $plugin_name The name of the plugin.
   * @param string  $version     The version of this plugin.
   *
   */
  public function __construct( $plugin_name, $version ) {

    $this->plugin_name = $plugin_name;
    $this->version = $version;

    // bring in some data
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/common_vars.php');
    $this->default_providers = $default_providers;
    $this->default_settings = $default_settings;
    $this->default_marker = $default_marker;
    $this->default_geocoder_services = $default_geocoder_services;
    $this->default_settings_mapped = $default_settings_mapped;
    $this->map_plugins = get_option('getlocations_lite_plugins');
    $this->bool = $bool;
    $this->default_leaflet_plugin_settings = $default_leaflet_plugin_settings;
    $this->default_leaflet_plugin_keys = $default_leaflet_plugin_keys;
    $this->plugins_bool = $plugins_bool;

    // bring in some data
    $mp  = get_option('getlocations_lite_providers', $default_providers);
    $map_key_data = get_option('getlocations_lite_map_keys');
    // ensure that the api key is brought in to map providers
    if ( isset($map_key_data['Thunderforest']) ) {
      $thunderforest_key = $map_key_data['Thunderforest'];
    }
    include(GETLOCATIONS_LITE_PLUGIN_DIR . 'admin/includes/providers.php');
    foreach ( $mp AS $k => $v  ) {
      if ( isset($default_leaflet_providers_data[$k]) && $v == '1' ) {
        $mp[$k] = $default_leaflet_providers_data[$k];
      }
    }
    $this->map_providers = $mp;
  }

  /**
   * Register the stylesheets for the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_styles() {

    /**
     * An instance of this class should be passed to the run() function
     * defined in Getlocations_Lite_Loader as all of the hooks are defined
     * in that particular class.
     *
     * The Getlocations_Lite_Loader will then create the relationship
     * between the defined hooks and the functions defined in this
     * class.
     */

    wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet-css', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/leaflet/leaflet.min.css', array(), '1.6.0', 'all' );
    if ( isset($this->map_plugins['leaflet_plugin_fullscreen']) && $this->map_plugins['leaflet_plugin_fullscreen'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_fullscreen', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.fullscreen/Control.FullScreen.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_mouseposition']) && $this->map_plugins['leaflet_plugin_mouseposition'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_mouseposition', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MousePosition/L.Control.MousePosition.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_pancontrol']) && $this->map_plugins['leaflet_plugin_pancontrol'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_pancontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Pancontrol/L.Control.Pan.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_zoomslider']) && $this->map_plugins['leaflet_plugin_zoomslider'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_zoomslider', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.zoomslider/L.Control.Zoomslider.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_magnifyingglass']) && $this->map_plugins['leaflet_plugin_magnifyingglass'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_magnifyingglass', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MagnifyingGlass/css/leaflet.magnifyingglass.min.css', array(), $this->version, 'all' );
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_magnifyingglasscontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MagnifyingGlass/css/Control.MagnifyingGlass.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_minimap']) && $this->map_plugins['leaflet_plugin_minimap'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_minimap', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.layerscontrol-minimap/control.layers.minimap.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_graticule']) && $this->map_plugins['leaflet_plugin_graticule'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_graticule', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.latlng-graticule/graticule.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_markercluster']) && $this->map_plugins['leaflet_plugin_markercluster'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_markercluster', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.markercluster/MarkerCluster.min.css', array(), $this->version, 'all' );
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_markercluster_default', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.markercluster/MarkerCluster.Default.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_terminator']) && $this->map_plugins['leaflet_plugin_terminator'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_terminator', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Terminator/terminator.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_awesome']) && $this->map_plugins['leaflet_plugin_awesome'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_awesome_markers', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.awesome-markers/leaflet.awesome-markers.min.css', array(), $this->version, 'all' );
      wp_enqueue_style( GETLOCATIONS_COMMON . '-ion', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/'  . 'css/ionicons.min.css', array(), $this->version, 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_geocoder']) && $this->map_plugins['leaflet_plugin_geocoder'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_geocoder', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet-control-geocoder/Control.Geocoder.min.css ', array(), '1.13.0', 'all' );
    }
    if ( isset($this->map_plugins['leaflet_plugin_kml']) && $this->map_plugins['leaflet_plugin_kml'] ) {
      wp_enqueue_style( GETLOCATIONS_COMMON . '-leaflet_plugin_kml', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.kml/kml.min.css', array(), $this->version, 'all' );
    }
    wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/getlocations-lite-public.min.css', array(), $this->version, 'all' );
  }


  /**
   * Register the JavaScript for the public-facing side of the site.
   *
   * @since    1.0.0
   */
  public function enqueue_scripts() {

    /**
     *
     * An instance of this class should be passed to the run() function
     * defined in Getlocations_Lite_Loader as all of the hooks are defined
     * in that particular class.
     *
     * The Getlocations_Lite_Loader will then create the relationship
     * between the defined hooks and the functions defined in this
     * class.
     */

    wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet-js', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/leaflet/leaflet.js' , array(), '1.6.0', FALSE );
    wp_enqueue_script( GETLOCATIONS_COMMON . '-providers-js', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/leaflet-providers/leaflet-providers.js', array(), '1.10.1', FALSE );

    // Plugins
    if ( isset($this->map_plugins['leaflet_plugin_fullscreen']) && $this->map_plugins['leaflet_plugin_fullscreen'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_fullscreen', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.fullscreen/Control.FullScreen.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_mouseposition']) && $this->map_plugins['leaflet_plugin_mouseposition'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_mouseposition', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MousePosition/L.Control.MousePosition.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_pancontrol']) && $this->map_plugins['leaflet_plugin_pancontrol'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_pancontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Pancontrol/L.Control.Pan.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_zoomslider']) && $this->map_plugins['leaflet_plugin_zoomslider'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_zoomslider', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.zoomslider/L.Control.Zoomslider.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_magnifyingglass']) && $this->map_plugins['leaflet_plugin_magnifyingglass'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_magnifyingglass', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MagnifyingGlass/js/leaflet.magnifyingglass.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_magnifyingglasscontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.MagnifyingGlass/js/Control.MagnifyingGlass.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_minimap']) && $this->map_plugins['leaflet_plugin_minimap'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_minimap', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.layerscontrol-minimap/L.Control.Layers.Minimap.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_graticule']) && $this->map_plugins['leaflet_plugin_graticule'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_graticule', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.latlng-graticule/leaflet.latlng-graticule.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_graticulecontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.latlng-graticule/Control.LatLngGraticule.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_markercluster']) && $this->map_plugins['leaflet_plugin_markercluster'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_markercluster', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.markercluster/leaflet.markercluster.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_terminator']) && $this->map_plugins['leaflet_plugin_terminator'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_terminator', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Terminator/L.Terminator.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_terminatorcontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.Terminator/Control.Terminator.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_awesome']) && $this->map_plugins['leaflet_plugin_awesome'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_awesome_markers', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/Leaflet.awesome-markers/leaflet.awesome-markers.min.js', array(), $this->version, FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_geocoder']) && $this->map_plugins['leaflet_plugin_geocoder'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_geocoder', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet-control-geocoder/Control.Geocoder.min.js', array(), '1.13.0', FALSE );
    }
    if ( isset($this->map_plugins['leaflet_plugin_kml']) && $this->map_plugins['leaflet_plugin_kml'] ) {
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_kml', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.kml/L.KML.min.js', array(), $this->version, FALSE );
      wp_enqueue_script( GETLOCATIONS_COMMON . '-leaflet_plugin_kmlcontrol', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/plugins/leaflet.kml/Control.Kml.min.js', array(), $this->version, FALSE );
    }
    wp_enqueue_script( $this->plugin_name . '-geo', GETLOCATIONS_LITE_PLUGIN_DIR_URL . 'public/js/getlocations-lite-geo.min.js', array( 'jquery' ), $this->version, FALSE );
    wp_enqueue_script( $this->plugin_name . '-public', plugin_dir_url( __FILE__ ) . 'js/getlocations-lite-public.min.js', array( 'jquery' ), $this->version, FALSE );
  }

  /**
   * output for shortcode getlocations_lite_map
   *
   * @return string
   */
  public function map_func( $atts, $content = null ){
    // bring in some data
    include_once(GETLOCATIONS_LITE_PLUGIN_DIR . 'includes/class-getlocations-lite-share.php');
    $share = new Getlocations_Lite_Share();

    static $getlocations_lite_map_id;
    if ( empty( $getlocations_lite_map_id ) ) {
      $getlocations_lite_map_id =1;
    }
    else {
      $getlocations_lite_map_id++;
    }

    // get the defaults and apply them to the available shortcode attributes
    $default_opts = $share->build_map_opts();
    $plugins_settings = $share->build_map_plugins_opts();

    $shortcode_defaults = $default_opts;
    $shortcode_defaults['map_name'] = '';
    $shortcode_defaults['map_id'] = '';
    if ( count($plugins_settings) ) {
      array_push($shortcode_defaults, $plugins_settings);
    }

    // active maps
    foreach ( $this->map_providers AS $n => $m ) {
      $shortcode_defaults['layer_' . strtolower($n)] = $m;
    }

    $a = shortcode_atts(
      $shortcode_defaults,
      $atts
    );

    $map_opts = $default_opts;
    $map_plugins_opts = $plugins_settings;

    // shortcode markers here....
    if ( $content ) {

      // $content may contain more than one getlocations_lite_marker
      // remove any linefeeds and carriage returns
      $content = preg_replace("/\n/", '', $content);
      $content = preg_replace("/\r/", '', $content);
      // split on "][", this returns an array to iterate over
      $ca = explode("][", $content);
      foreach ($ca AS $k => $t) {
        $t = preg_replace("/^\[/", '', $t);
        $t = preg_replace("/\]$/", '', $t);
        if ( preg_match( "/getlocations_lite_marker/", $t ) ) {
          $js = do_shortcode('['.$t.']');
          $markers[] = json_decode($js, TRUE);
        }
        elseif ( preg_match( "/getlocations_lite_plugins/", $t ) ) {
          $js = do_shortcode('['.$t.']');
          // integrate with saved map
          $mpo = json_decode($js, TRUE);
          $mpoc = $share->convert_plugins_keys($mpo);
          $map_plugins_opts = array_replace_recursive($map_plugins_opts, $mpoc);
        }
        #else {
        #  $map_plugins_opts = array();
        #}
      }
    } // end content

    $shortcode_defaults = array_replace_recursive($shortcode_defaults, $map_opts);
    $a = shortcode_atts(
      $shortcode_defaults,
      $atts
    );

    // merge shortcode options with map options
    $map_opts2 = array();
    foreach ( $this->default_settings_mapped AS $sh_key => $mp_key ) {

      if ( isset( $a[$sh_key] ) && strlen($a[$sh_key]) > 0 ) {
        $map_opts2[$sh_key] = $a[$sh_key];
      }
      elseif ( isset( $map_opts[$sh_key] ) ) {
        $map_opts2[$sh_key] = $map_opts[$sh_key];
      }
    }

    // layers_ are missing in $map_opts2

    $map_layers = array();
    foreach ( $map_opts AS $k => $v ) {
      if ( preg_match("/^layers_/", $k) && $v ) {
        $l = preg_replace("/^layers_/", '', $k);
        $l = preg_replace("/_/", '.', $l);
        $map_layers[$l] = $this->map_providers[$l];
      }
    }
    // if there is nothing there set the default
    if ( count($map_layers) < 1 ) {
      $l = $this->default_settings['map_provider'];
      $map_layers[$l] = $this->map_providers[$l];
    }
    // get the getlocations_lite_providers and top up $map_layers
    $getlocations_lite_providers = get_option('getlocations_lite_providers', $this->default_settings['map_provider']);
    foreach ( $getlocations_lite_providers AS $k => $v ) {
      if (! isset( $map_layers[$k] ) && $v ) {
        $map_layers[$k] = $this->map_providers[$k];
      }
    }

    #$map_opts = $map_opts2;
    $map_opts = array_replace_recursive($map_opts, $map_opts2);

    if ( isset($map_opts['layers']) ) {
      unset($map_opts['layers']);
    }

    if (! isset($markers) ) {
      $markers = array();
      $map_opts['minmaxes'] = '';
    }

    // calculate minmaxes from all the markers latlons, if any
    // get them first
    $minmaxes = '';
    if ( is_array($markers) && count($markers) ) {
      $ct = 0;
      foreach ($markers AS $m) {
        $latlon = $share->latlon_check($m['latlon']);
        if ( $latlon ) {
          $sla = explode(",", $latlon);
          $location = array('latitude' => $sla[0], 'longitude' => $sla[1]);
          $minmaxes = $share->do_minmaxes($ct, $location, $minmaxes);
          $ct++;
        }
      }
    }
    // add minmaxes to $map_opts as a string
    $map_opts['minmaxes'] = (empty($minmaxes) ? FALSE : implode(',', $minmaxes));

    $map_opts2 = $share->get_map_opts_mapped($map_opts);

    $map_opts_arr = array(
      'map_opts' => $map_opts2,
      'map_layers' => $map_layers,
      'markers' => $markers,
      'map_plugins_opts' => $map_plugins_opts
    );
    $map_opts_js = json_encode($map_opts_arr);

    $inline_data = '';
    if ( $getlocations_lite_map_id < 2 ) {
      $inline_data .= 'var getlocations_lite_settings = [];'."\n";
    }
    $inline_data .= 'getlocations_lite_settings["key_'.$getlocations_lite_map_id.'"] = '.$map_opts_js.';'."\n";
    $this->map_func_enqueue_inline($inline_data);
    $c = $share->get_marker_script($markers, $getlocations_lite_map_id);
    $this->map_func_enqueue_inline($c);
    add_action('wp_enqueue_scripts', 'map_func_enqueue_inline');

    $template_path = $this->process_template('map.php');
    ob_start();
    include($template_path);
    $output = ob_get_clean();
    return $output;

  }

  /**
   * markers shortcode
   * @param array $atts
   * @param string $content
   * @return string json
   *
   * marker_type: ticon licon vicon dicon
   * icon_type: awesome dashicon ionicon
   * awesome requires icon_category: brand solid regular
   * icon_selected: fontawesome example 'fas fa-address-book'
   * icon_color used with vicon dicon
   * icon_effect
   * marker_color is used with licon and vicon
   */
  public function marker_func( $atts, $content = NULL ) {
    $shortcode_defaults = array(
      'marker_type' => 'ticon',
      'icon' => $this->default_marker,
      'latlon' => $this->default_settings['mapcenter'],
      'title' => '',
      'marker_color' => 'blue',
      'icon_type' => 'awesome',
      'icon_selected' => 'fas fa-address-book',
      'icon_effect' => '',
      'icon_size' => '',
      'icon_color' => 'White',
    );
    $a = shortcode_atts(
      $shortcode_defaults,
      $atts
    );

    $shortcode_keys_mapped = array(
      'marker_type' => 'marker_type',
      'icon' => 'icon',
      'latlon' => 'latlon',
      'title' => 'title',
      'marker_color' => 'marker_color',
      'icon_type' => 'icon_type',
      'icon_selected' => 'icon_selected',
      'icon_effect' => 'icon_effect',
      'icon_size' => 'icon_size',
      'icon_color' => 'icon_color',
    );
    // provide an array
    $marker = array();
    foreach ( $shortcode_keys_mapped AS $sh_key => $m_key ) {
      if ( isset( $a[$sh_key] ) && ! empty( $a[$sh_key] ) ) {
        $marker[$m_key] = $a[$sh_key];
      }
    }
    // need to delete some params according to marker_type
    // title is always allowed, latlon is always required
    $markertemp = array();
    $markertemp['latlon'] = $marker['latlon'];
    $markertemp['title'] = (isset($marker['title']) ? $marker['title'] : '' );
    $markertemp['marker_type'] = $marker['marker_type'];

    // dicon
    if ( $marker['marker_type'] == 'dicon' ) {
      // icon only : icon_type
      #$markertemp[''] = $marker[''];
      $markertemp['icon_type'] = $marker['icon_type'];
      $markertemp['icon_selected'] = $marker['icon_selected'];
      $markertemp['icon_color'] = $marker['icon_color'];
      if ( isset($marker['icon_size']) && $marker['icon_size'] ) {
        $markertemp['icon_size'] = $marker['icon_size'];
      }
      if ( isset($marker['icon_effect']) && $marker['icon_effect'] ) {
        $markertemp['icon_effect'] = $marker['icon_effect'];
      }
    }
    elseif ( $marker['marker_type'] == 'licon' ) {
      // leaflet marker only
      $markertemp['marker_color'] = $marker['marker_color'];
    }
    elseif ( $marker['marker_type'] == 'vicon' ) {
      // icon in a leaflet marker
      $markertemp['marker_color'] = $marker['marker_color'];
      $markertemp['icon_type'] = $marker['icon_type'];
      $markertemp['icon_selected'] = $marker['icon_selected'];
      $markertemp['icon_color'] = $marker['icon_color'];
      if ( isset($marker['icon_effect']) && $marker['icon_effect'] ) {
        $markertemp['icon_effect'] = $marker['icon_effect'];
      }
    }
    else {
      // ticon default
      $markertemp['icon'] = $marker['icon'];
      $markertemp['marker_type'] = 'ticon';
    }
    $marker = $markertemp;

    // set content to be a popup
    if ( $content ) {
      $marker['popupContent'] = $content;
    }
    $marker_js = json_encode( $marker );
    return $marker_js;

  }

  /**
   * plugins shortcode
   * @param array $atts
   * @param string $content
   * @return string json
   */
  public function plugins_func( $atts, $content = NULL ) {

    $plugin_js = json_encode( $atts );
    return $plugin_js;

  }

  /**
   * insert js into DOM via enqueue system
   * @param string $js
   *
   */
  private function map_func_enqueue_inline($js) {
    wp_enqueue_script( $this->plugin_name . '-inline', plugin_dir_url( __FILE__ ) . 'js/inline.js', array(), $this->version, TRUE );
    wp_add_inline_script( $this->plugin_name . '-inline', $js );
  }

  /**
   * allow themes to override templates
   *
   */
  private function process_template($file) {
    $themepath = get_stylesheet_directory();
    if ( file_exists( $themepath .'/'. GETLOCATIONS_LITE_PLUGIN_SLUG . '/public/' . $file ) ) {
      $path = $themepath .'/'. GETLOCATIONS_LITE_PLUGIN_SLUG . '/public/' . $file;
    }
    else {
      $path = GETLOCATIONS_LITE_PLUGIN_PUBLIC_TEMPLATES_DIR . $file;
    }
    return $path;
  }

}
