/**
 * @file    getlocations-lite-public.js
 * @author  Bob Hutchinson
 * @package Getlocations_Lite
 * @since   1.0.0
 */

if ( typeof(getlocations_lite) == 'undefined') {
  getlocations_lite = {};
}

jQuery(document).ready(function($) {

  'use strict';

  var Markers = [];
  var icons = [];
  var getlocations_lite_map = getlocations_lite_map || [];
  var mapkey = '' ;
  var getlocations_lite_leaflet_overlays = [];
  var getlocations_lite_leaflet_layerscontrol = [];
  var markersGroup = [];
  var allIconInfo = [];

  // Map, one or more
  $(".getlocations_lite_map_canvas").each( function(index) {

    var elemID = $(this).attr('id');
    mapkey = elemID.replace(/^getlocations_lite_map_canvas_/, '');
    // is there really a map?
    if ( $("#getlocations_lite_map_canvas_" + mapkey).is('div') ) {

      var layers = [];
      getlocations_lite_leaflet_overlays[mapkey] = {};
      getlocations_lite_leaflet_layerscontrol[mapkey] = {};
      //data from the DOM
      var map_opts = getlocations_lite_settings[mapkey]['map_opts'];
      var map_layers = getlocations_lite_settings[mapkey]['map_layers'];
      var inmarkers = getlocations_lite_settings[mapkey]['markers'];
      var map_plugins_opts = getlocations_lite_settings[mapkey]['map_plugins_opts'];
      // Get allIconInfo if available
      if ( getlocations_lite_iconinfo !== undefined ) {
        allIconInfo = getlocations_lite_iconinfo[mapkey];
      }
      // set up the center
      map_opts.center = get_latlon(map_opts['mapcenter']);
      var default_layer_name = map_opts.map_provider;
      var map_layer = L.tileLayer.provider(default_layer_name);
      var default_layer_label = map_layers[default_layer_name].label;
      // bring in apikey
      if ( map_layers[default_layer_name].group == "Thunderforest" ) {
        map_layer.options.apikey = map_layers[default_layer_name].options.apikey;
      }
      map_opts.layers = map_layer;
      // fetch maxZoom from providers.js and apply
      if (map_layer.options.maxZoom) {
        map_opts.maxZoom = map_layer.options.maxZoom;
      }
      if (map_layer.options.minZoom) {
        map_opts.minZoom = map_layer.options.minZoom;
      }
      // force type
      map_opts.minZoom = parseInt(map_opts.minZoom);
      map_opts.maxZoom = parseInt(map_opts.maxZoom);
      map_opts.zoom = parseInt(map_opts.zoom);
      // grab zoom control if position is not the default
      var zoom_opt = false;
      if ( map_opts.zoomcontrol_position !== 'topleft' ) {
        zoom_opt = map_opts.zoomControl;
        map_opts.zoomControl = false;
      }

      // get the map
      getlocations_lite_map[mapkey] = L.map('getlocations_lite_map_canvas_' + mapkey, map_opts);
      // add the default map
      layers[default_layer_label] = map_layer.addTo(getlocations_lite_map[mapkey]);
      if (layers.length) {
        layers.addTo(getlocations_lite_map[mapkey]);
      }

      // do zoomControl
      if ( zoom_opt ) {
        L.control.zoom({ position: map_opts.zoomcontrol_position }).addTo(getlocations_lite_map[mapkey]);
      }

      // Fullscreen button
      if ( map_plugins_opts.fullscreen !== undefined ) {
        if ( map_plugins_opts.fullscreen.enable ) {
          var fsOpts = {};
          if (map_plugins_opts.fullscreen.position) {
            fsOpts.position = map_plugins_opts.fullscreen.position;
          }
          if ( map_plugins_opts.fullscreen.forceseparatebutton ) {
            fsOpts.forceSeparateButton = true;
          }
          else {
            fsOpts.forceSeparateButton = false;
          }
          // title
          if ( map_plugins_opts.fullscreen.label ) {
            fsOpts.title = map_plugins_opts.fullscreen.label;
          }
          // titleCancel
          if ( map_plugins_opts.fullscreen.cancel_label ) {
            fsOpts.titleCancel = map_plugins_opts.fullscreen.cancel_label;
          }
          getlocations_lite_map[mapkey].addControl(L.control.fullscreen(fsOpts));
        }
      }

      // Mouseposition
      if ( map_plugins_opts.mouseposition !== undefined ) {
        if ( map_plugins_opts.mouseposition.enable) {
          var plopts = {};
          plopts.emptystring = 'Unavailable';
          if (map_plugins_opts.mouseposition.position) {
            plopts.position = map_plugins_opts.mouseposition.position;
          }
          if (map_plugins_opts.mouseposition.display_dms) {
            plopts.lngFormatter = getlocations_lite.geo.dd_to_dms_lng;
            plopts.latFormatter = getlocations_lite.geo.dd_to_dms_lat;
            plopts.display_dms = true;
          }
          else {
            plopts.lngFormatter = getlocations_lite.geo.dd_lng;
            plopts.latFormatter = getlocations_lite.geo.dd_lat;
            plopts.display_dms = false;
          }
          if ( map_plugins_opts.mouseposition.emptystring ) {
            plopts.emptyString  = map_plugins_opts.mouseposition.emptystring;
          }
          if ( map_plugins_opts.mouseposition.separator ) {
            plopts.separator  = map_plugins_opts.mouseposition.separator;
          }
          if ( map_plugins_opts.mouseposition.lngfirst ) {
            plopts.lngFirst  = map_plugins_opts.mouseposition.lngfirst;
          }
          if ( map_plugins_opts.mouseposition.numdigits ) {
            plopts.numDigits  = parseInt(map_plugins_opts.mouseposition.numdigits);
          }
          if ( map_plugins_opts.mouseposition.prefix ) {
            plopts.prefix  = map_plugins_opts.mouseposition.prefix;
          }
          getlocations_lite_map[mapkey].addControl(L.control.mousePosition(plopts));
        }
      }

      // pancontrol
      if ( map_plugins_opts.pancontrol !== undefined ) {
        if ( map_plugins_opts.pancontrol.enable ) {
          var plopts = {};
          if (map_plugins_opts.pancontrol.position) {
            plopts.position = map_plugins_opts.pancontrol.position;
          }
          getlocations_lite_map[mapkey].addControl(L.control.pan(plopts));
        }
      }

      // zoomslider
      if ( map_plugins_opts.zoomslider !== undefined ) {
        if ( map_plugins_opts.zoomslider.enable ) {
          var plopts = {};
          if (map_plugins_opts.zoomslider.position) {
            plopts.position = map_plugins_opts.zoomslider.position;
          }
          getlocations_lite_map[mapkey].addControl(L.control.zoomslider(plopts));
        }
      }

      // magnifyingglass
      if ( map_plugins_opts.magnifyingglass !== undefined ) {
        if ( map_plugins_opts.magnifyingglass.enable ) {
          var la =  L.tileLayer.provider(default_layer_name);
          var magOpts = {
            zoomOffset: parseInt(map_plugins_opts.magnifyingglass.offset),
            radius: parseInt(map_plugins_opts.magnifyingglass.radius),
            fixedPosition: false,
            latLng: [0, 0],
            fixedZoom: -1,
            layers: [L.tileLayer(la._url, la.options)]
          };
          var magnifyingGlass = L.magnifyingGlass(magOpts);
          var magctlOpts = {};
          if ( map_plugins_opts.magnifyingglass.forceseparatebutton ) {
            magctlOpts.forceSeparateButton = true;
          }
          else {
            magctlOpts.forceSeparateButton = false;
          }
          if ( map_plugins_opts.magnifyingglass.position ) {
            magctlOpts.position = map_plugins_opts.magnifyingglass.position;
          }
          if ( map_plugins_opts.magnifyingglass.label ) {
            magctlOpts.title = map_plugins_opts.magnifyingglass.label;
          }
          getlocations_lite_map[mapkey].addControl( L.control.magnifyingglass(magnifyingGlass, magctlOpts) );
        }
      }

      // scale
      if ( map_opts.scale !== undefined ) {
        if ( map_opts.scale ) {
          var scOpts = {};
          if ( map_opts.scale_position ) {
            scOpts.position = map_opts.scale_position;
          }
          if (map_opts.scaleunit) {
            if (map_opts.scaleunit == 'metric') {
              scOpts.metric = true;
              scOpts.imperial = false;
            }
            else if (map_opts.scaleunit == 'imperial') {
              scOpts.metric = false;
              scOpts.imperial = true;
            }
          }
          getlocations_lite_map[mapkey].addControl(L.control.scale(scOpts));
        }
      }

      // latlngGraticule
      if ( map_plugins_opts.graticule !== undefined ) {
        if ( map_plugins_opts.graticule.enable ) {
          var gropts = {
            showLabel: map_plugins_opts.graticule.linelabel,
            opacity: map_plugins_opts.graticule.line_opacity,
            weight: map_plugins_opts.graticule.line_thickness ,
            color: map_plugins_opts.graticule.line_color,
            font: map_plugins_opts.graticule.font,
            fontColor: map_plugins_opts.graticule.font_color
          };
          if ( map_plugins_opts.graticule.zoomInterval !== undefined ) {
            gropts.zoomInterval = {
              latitude: map_plugins_opts.graticule.zoomInterval.latitude,
              longitude: map_plugins_opts.graticule.zoomInterval.longitude
            };
          }
          var graticule = L.latlngGraticule(gropts);
          var gratctlOpts = {};
          if ( map_plugins_opts.graticule.forceseparatebutton ) {
            gratctlOpts.forceSeparateButton = true;
          }
          else {
            gratctlOpts.forceSeparateButton = false;
          }
          if ( map_plugins_opts.graticule.position ) {
            gratctlOpts.position = map_plugins_opts.graticule.position;
          }
          if ( map_plugins_opts.graticule.label ) {
            gratctlOpts.title = map_plugins_opts.graticule.label;
          }
          // showstate
          if ( map_plugins_opts.graticule.show == 'button' || map_plugins_opts.graticule.show == 'both' ) {
            getlocations_lite_map[mapkey].addControl(L.control.latlngGraticule(graticule, gratctlOpts));
          }
          else {
            getlocations_lite_map[mapkey].addControl(graticule);
          }
          //switch on
          if ( map_plugins_opts.graticule.initial_state ) {
            graticule.addTo(getlocations_lite_map[mapkey]);
          }
        }
      }

      // markercluster
      var marker_groups_enable = map_opts.marker_groups_enable;
      var marker_groups = map_opts.marker_groups;
      // markergroups without clustering
      markersGroup['all'] =  L.layerGroup();
      if ( marker_groups_enable ) {
        for (var c in marker_groups ) {
          markersGroup[ c ] =  L.layerGroup();
        }
      }
      if ( map_plugins_opts.markercluster !== undefined ) {
        if ( map_plugins_opts.markercluster.enable ) {
          // markerclusteroptions
          var markerclusteroptions = {};
          if ( map_plugins_opts.markercluster.markerclusteroptions !== undefined ) {
            for ( var k in map_plugins_opts.markercluster.markerclusteroptions ) {
              markerclusteroptions[k] = map_plugins_opts.markercluster.markerclusteroptions[k];
            }
          }
          // marker_groups
          markersGroup['all'] =  L.markerClusterGroup(markerclusteroptions);
        }
      }

      // Terminator or Day/Night
      if (map_plugins_opts.terminator !== undefined ) {
        if ( map_plugins_opts.terminator.enable ) {

          var terminatorOpts = {
            color: map_plugins_opts.terminator.line_color,
            opacity: map_plugins_opts.terminator.line_opacity,
            weight: map_plugins_opts.terminator.line_thickness,
            fillColor: map_plugins_opts.terminator.fill_color,
            fillOpacity: map_plugins_opts.terminator.fill_opacity,
            resolution: 2
          };
          var terminator = L.terminator(terminatorOpts);
          var termctlOpts = {};
          if ( map_plugins_opts.terminator.position ) {
            termctlOpts.position = map_plugins_opts.terminator.position;
          }
          if ( map_plugins_opts.terminator.forceseparatebutton ) {
            termctlOpts.forceSeparateButton = true;
          }
          else {
            termctlOpts.forceSeparateButton = false;
          }
          termctlOpts.title = map_plugins_opts.terminator.label;
          if ( map_plugins_opts.terminator.show == 'button' || map_plugins_opts.terminator.show == 'both' ) {
            getlocations_lite_map[mapkey].addControl(L.control.terminator(terminator, termctlOpts));
          }
          else {
            getlocations_lite_map[mapkey].addControl(terminator);
          }

          // on to start
          if ( map_plugins_opts.terminator.initial_state ) {
            terminator.addTo(getlocations_lite_map[mapkey]);
          }
        }
      }

      // Geocoder
      if (map_plugins_opts.geocoder !== undefined ) {
        if ( map_plugins_opts.geocoder.enable ) {
          var gopts = {};
          if ( map_plugins_opts.geocoder.position ) {
            gopts.position = map_plugins_opts.geocoder.position;
          }
          if ( map_plugins_opts.geocoder.placeholder ) {
            gopts.placeholder = map_plugins_opts.geocoder.placeholder;
          }
          if ( map_plugins_opts.geocoder.label ) {
            gopts.title = map_plugins_opts.geocoder.label;
          }
          // we have a selected service (or should have)
          var geoservice = 'nominatim';
          var geosettings = {};
          var geokey = '';
          var userid = '';
          if ( map_plugins_opts.geocoder.service ) {
            geoservice = map_plugins_opts.geocoder.service;
          }
          if ( map_plugins_opts.geocoder.settings ) {
            geosettings = map_plugins_opts.geocoder.settings;
          }
          // testing this, icons appear in the search suggestions dd
          // only for nominatim apparently
          var nopts = {};
          if ( geoservice == 'nominatim' ) {
            gopts.showResultIcons = true;
          }
          // geosettings
          if ( geosettings ) {
            if ( geosettings.needkey && geosettings.key !== '' ) {
              geokey = geosettings.key;
            }
            // userid (neutrino)
            if ( geosettings.userId && geosettings.apiKey ) {
              //userid = geosettings.userid;
              nopts.userId = geosettings.userId;
              nopts.apiKey = geosettings.apiKey;
            }
            if ( geosettings.countrycodes && map_plugins_opts.geocoder.country ) {
              if ( geoservice == 'nominatim' ) {
                nopts.geocodingQueryParams = {countrycodes: map_plugins_opts.geocoder.country};
              }
              else if ( geoservice == 'arcgis' ) {
                nopts.geocodingQueryParams = {countryCode: map_plugins_opts.geocoder.country};
              }
              else if ( geoservice == 'opencage' ) {
                nopts.geocodingQueryParams = {countrycode: map_plugins_opts.geocoder.country};
              }
              else if ( geoservice == 'neutrino' ) {
                nopts.geocodingQueryParams = {"country-code": map_plugins_opts.geocoder.country};
              }
              else if ( geoservice == 'mapbox' ) {
                nopts.geocodingQueryParams = {country: map_plugins_opts.geocoder.country};
              }
            }
            if ( geosettings.label ) {
              gopts.title = gopts.title + ' by ' + geosettings.label;
            }
          }
          if ( geokey ) {
            gopts.geocoder = L.Control.Geocoder[ geoservice ](geokey, nopts);
          }
          else {
            gopts.geocoder = L.Control.Geocoder[ geoservice ](nopts);
          }

          // other search engines
          // arcgis(accessToken, options) WORKS, with cc
          // bing requires key
          // google(key, options) NEEDS TO BE TESTED on a real domain
          // here requires app_id and app_code
          // latLng returns place from lat lng
          // mapbox(accessToken, options) WORKS, with cc
          // mapQuest(key, options) NEEDS TO BE TESTED on a real domain
          // neutrino userid and apikey neutrino(accessToken)
          // nominatim(options) default
          // openLocationCode(options) ?
          // opencage(apiKey) WORKS, with cc
          // pelias(apiKey, options) appears to be dead or not free to dev on api.geocode.earth
          // photon(options) WORKS, no country limiter AFAICT
          // what3words(accessToken)

          getlocations_lite_map[mapkey].addControl(L.Control.geocoder(gopts));
          if (geosettings.reverse && map_plugins_opts.geocoder.click ) {
            var rmarker;
            getlocations_lite_map[mapkey].on('click', function(e) {
              if ( geokey ) {
                var geocoder = L.Control.Geocoder[ geoservice ](geokey,{});
              }
              else {
                var geocoder = L.Control.Geocoder[ geoservice ]();
              }
              geocoder.reverse(e.latlng, getlocations_lite_map[mapkey].options.crs.scale(getlocations_lite_map[mapkey].getZoom()), function(results) {
                var r = results[0];
                if (r) {
                  if (rmarker) {
                    rmarker
                    .setLatLng(r.center)
                    .setPopupContent(r.html || r.name)
                    .openPopup();
                  } else {
                    rmarker = L.marker(r.center)
                    .bindPopup(r.name)
                    .addTo(getlocations_lite_map[mapkey])
                    .openPopup();
                  }
                }
              });
            });
          }
        }
      }

      // KML
      if (map_plugins_opts.kml !== undefined ) {
        if ( map_plugins_opts.kml.enable && map_plugins_opts.kml.url ) {
          // fetch url
          fetch(map_plugins_opts.kml.url)
          .then(res => res.text())
          .then(kmltext => {
            // Create new kml overlay
            var parser = new DOMParser();
            var kml = parser.parseFromString(kmltext, 'text/xml');
            var kmlLayer = new L.KML(kml);
            var kmlctlOpts = {};
            if ( map_plugins_opts.kml.forceseparatebutton ) {
              kmlctlOpts.forceSeparateButton = true;
            }
            else {
              kmlctlOpts.forceSeparateButton = false;
            }
            if ( map_plugins_opts.kml.position ) {
              kmlctlOpts.position = map_plugins_opts.kml.position;
            }
            if ( map_plugins_opts.kml.label ) {
              kmlctlOpts.title = map_plugins_opts.kml.label;
            }
            // enable_button
            if ( map_plugins_opts.kml.enable_button ) {
              getlocations_lite_map[mapkey].addControl(L.control.kml(kmlLayer, kmlctlOpts));
            }
            //switch on
            if ( map_plugins_opts.kml.initial_state ) {
              kmlLayer.addTo(getlocations_lite_map[mapkey]);
              //getlocations_lite_map[mapkey].addLayer(kmlLayer);
            }
            // Adjust map to show the kml
            if ( map_plugins_opts.kml.viewport ) {
              var kmlbounds = kmlLayer.getBounds();
              getlocations_lite_map[mapkey].invalidateSize().fitBounds(kmlbounds, {reset: true});
            }
          });
        }
      }

      // more plugins here

      // these crash minimap
      if (! map_opts.minimap ) {
        // adding graticule to overlay switches
        if (map_plugins_opts.graticule && map_plugins_opts.graticule.enable ) {
          if ( map_plugins_opts.graticule.show == 'layer' || map_plugins_opts.graticule.show == 'both' ) {
            getlocations_lite_leaflet_overlays[mapkey][map_plugins_opts.graticule.label] = graticule;
            //switch on
            if ( map_plugins_opts.graticule.initial_state ) {
              getlocations_lite_leaflet_overlays[mapkey][map_plugins_opts.graticule.label].addTo(getlocations_pro_map[mapkey]);
            }
            else {
              getlocations_lite_leaflet_overlays[mapkey][map_plugins_opts.graticule.label].remove();
            }
          }
        }

        // adding terminator to overlay switches
        if (map_plugins_opts.terminator && map_plugins_opts.terminator.enable ) {
          if ( map_plugins_opts.terminator.show == 'layer' || map_plugins_opts.terminator.show == 'both' ) {
            getlocations_lite_leaflet_overlays[mapkey][map_plugins_opts.terminator.label] = terminator;
            //switch on
            if ( map_plugins_opts.terminator.initial_state ) {
              getlocations_lite_leaflet_overlays[mapkey][map_plugins_opts.terminator.label].addTo(getlocations_pro_map[mapkey]);
            }
            else {
              getlocations_lite_leaflet_overlays[mapkey][map_plugins_opts.terminator.label].remove();
            }
          }
        }
      }

      // markers
      if ( inmarkers !== undefined &&  inmarkers.length ) {
        for ( var mct = 0; mct < inmarkers.length; mct++ ) {
          // RESET. much better :)
          var iconinfo = {};
          var mindex = mct+1;
          var popupContent = [];
          popupContent[mct] = (inmarkers[mct]['popupContent'] == undefined ? '' : inmarkers[mct]['popupContent']);
          // ticon
          if ( inmarkers[mct]['marker_type'] == 'ticon' ) {
            var datakey = inmarkers[mct]['icon'];
            var iconinfo = allIconInfo[datakey];
            var iconinfo = iconinfo_prepare(iconinfo);
            icons[mindex] = L.icon(iconinfo);
          }
          // licon
          else if ( inmarkers[mct]['marker_type'] == 'licon' ) {
            // kill icon
            iconinfo['icon'] = false;
            iconinfo['prefix'] = false;
            iconinfo['markerColor'] = inmarkers[mct]['marker_color'];
            iconinfo['iconUrl'] = undefined;
            delete iconinfo['iconUrl'];
            iconinfo['shadowUrl'] = undefined;
            delete iconinfo['shadowUrl'];
            iconinfo['version'] = '5';
            icons[mindex] = L.AwesomeMarkers.icon(iconinfo);
          }
          // vicon
          else if ( inmarkers[mct]['marker_type'] == 'vicon' ) {
            // make leaflet icon
            $("#icon_sel_" + mindex).val( inmarkers[mct]['icon_selected'] );
            var sarr = inmarkers[mct]['icon_selected'].split(' ');
            iconinfo['icon'] = sarr[1];
            iconinfo['prefix'] = sarr[0];
            var xtra = '';
            if ( inmarkers[mct]['icon_effect'] !== undefined && inmarkers[mct]['icon_effect'].length > 0 ) {
              iconinfo['spin'] = false;
              if ( inmarkers[mct]['icon_effect'] == 'fa-spin' || inmarkers[mct]['icon_effect'] == 'fa-pulse' ) {
                iconinfo['spin'] = true;
                iconinfo['spinClass'] = inmarkers[mct]['icon_effect'];
              }
              else {
                xtra += inmarkers[mct]['icon_effect'] + ' ';
              }
            }
            else {
              iconinfo['spinClass'] = '';
            }
            iconinfo['extraClasses'] = xtra;
            iconinfo['iconColor'] = inmarkers[mct]['icon_color'];
            iconinfo['markerColor'] = inmarkers[mct]['marker_color'];
            iconinfo['iconUrl'] = undefined;
            delete iconinfo['iconUrl'];
            iconinfo['shadowUrl'] = undefined;
            delete iconinfo['shadowUrl'];
            iconinfo['version'] = '5';
            icons[mindex] = L.AwesomeMarkers.icon(iconinfo);
          }
          // dicon
          else if ( inmarkers[mct]['marker_type'] == 'dicon' ) {
            // divicon icon only
            if ( inmarkers[mct]['icon_type'] == undefined ) {
              inmarkers[mct]['icon_type'] = 'awesome';
            }
            var icon_selected = '';
            if ( inmarkers[mct]['icon_type'] == 'awesome' ) {
              $("#icon_sel_" + mindex).val( inmarkers[mct]['icon_selected'] );
              icon_selected = inmarkers[mct]['icon_selected'];
            }
            else if ( inmarkers[mct]['icon_type'] == 'dashicon' ) {
              $("#dashicon_sel_" + mindex).val( inmarkers[mct]['icon_selected'] );
              icon_selected = inmarkers[mct]['icon_selected'];
            }
            else if ( inmarkers[mct]['icon_type'] == 'ionicon' ) {
              $("#ionicon_sel_" + mindex).val( inmarkers[mct]['icon_selected'] );
              icon_selected = inmarkers[mct]['icon_selected'];
            }
            if ( icon_selected !== '' || typeof icon_selected !== 'undefined' ) {
              var sarr = icon_selected.split(' ');
              iconinfo['icon'] = sarr[1];
              iconinfo['prefix'] = sarr[0];
              if ( inmarkers[mct]['icon_effect'] == undefined ) {
                inmarkers[mct]['icon_effect'] = '';
              }
              var iconColorStyle = "style='color: " + inmarkers[mct]['icon_color'] + "' ";
              var istr = '<i ' + iconColorStyle + ' class="' + iconinfo['prefix'] + ' ' + iconinfo['icon'] +' '+ inmarkers[mct]['icon_effect'] + ' '+ inmarkers[mct]['icon_size'] + '" ></i>';
              var diopts = {
                className: 'getlocations-pro-div-icon',
                iconSize: iconinfo['iconSize'],
                html: istr
              };
              icons[mindex] = L.divIcon(diopts);
            }
          }
          // actions common to all markers, latlon, title, popup
          var mpos_latlng = get_latlon(inmarkers[mct]['latlon']);
          var mmopts = {
            icon: icons[mindex],
            riseOnHover: true
          };
          if ( inmarkers[mct]['title'] ) {
            mmopts['title'] = inmarkers[mct]['title'];
          }
          // Make the marker
          Markers[mindex] = L.marker(mpos_latlng, mmopts);
          // add to 'all' group
          markersGroup['all'].addLayer( Markers[mindex] );
          // popup
          if ( popupContent[mct].length > 0 ) {
            Markers[mindex].bindPopup(popupContent[mct]).on('dblclick', function() {
              this.openPopup();
            });
          }
        } // end for loop
        // add all the grouped markers to the map
        getlocations_lite_map[mapkey].addLayer(markersGroup['all']);
        // fitBounds
        if ( map_opts.fitBounds ) {
          var minmaxes = map_opts.minmaxes;
          if (minmaxes) {
            var mmarr = minmaxes.split(',');
            var sw = L.latLng(parseFloat(mmarr[2]), parseFloat(mmarr[3])),
              ne = L.latLng(parseFloat(mmarr[0]), parseFloat(mmarr[1])),
              bounds = L.latLngBounds(sw, ne).pad(0.1);
              getlocations_lite_map[mapkey].invalidateSize().fitBounds(bounds, {reset: true});
          }
        }
      } // end inmarkers.length

      // map_layers
      if ( map_opts.layersControl !== undefined ) {
        if ( map_opts.layersControl) {
          for ( var l in map_opts ) {
            if ( l.match(/^layers_/) ) {
              var lkey = l.replace(/^layers_/, '');
              if (lkey != default_layer_name) {
                var layer = map_layers[lkey];
                var map_layer = L.tileLayer.provider(lkey);
                map_layer._leaflet_id = lkey;
                if ( layer ) {
                  for (var option in layer.options) {
                    map_layer.options[option] = layer.options[option];
                  }
                  if (layer.type == 'base') {
                    layers[layer.label] = map_layer;
                  }
                  else if (layer.type == 'overlay') {
                    getlocations_lite_leaflet_overlays[mapkey][layer.label] = map_layer;
                  }
                }
              }
            }
          }
          // layers control
          var layeropts = {};
          if (map_opts.layerscontrol_position) {
            layeropts = {
              position: map_opts.layerscontrol_position
            };
          }
          if ( map_opts.minimap ) {
            getlocations_lite_leaflet_layerscontrol[mapkey] = L.control.layers.minimap(layers, getlocations_lite_leaflet_overlays[mapkey], layeropts);
          }
          else {
            getlocations_lite_leaflet_layerscontrol[mapkey] = L.control.layers(layers, getlocations_lite_leaflet_overlays[mapkey], layeropts);
          }
          getlocations_lite_leaflet_layerscontrol[mapkey].addTo(getlocations_lite_map[mapkey]);
        }
      } // end layerscontrol
    } // is div
  }); // each

  /* functions */

  /**
   *
   * @param string latlons
   * @return latlons prepared for Leaflet
   */
  function get_latlon(latlons) {
    var latlons_ll  = latlons.split(',');
    var latlons_latlng = L.latLng(parseFloat(latlons_ll[0]),parseFloat(latlons_ll[1]));
    return latlons_latlng;
  }

  /**
   *
   * @param array iconinf
   */
  function iconinfo_prepare(iconinf) {
    var keylist = ["iconSize","iconAnchor","popupAnchor","tooltipAnchor","shadowSize","shadowAnchor"];
    for (var ic in iconinf ) {
      if ( keylist.includes( ic ) ) {
        var t = String( iconinf[ic] );
        t = t.split(",");
        iconinf[ic] = L.point(parseInt(t[0], 10), parseInt(t[1], 10));
      }
      else if ( ic == "iconUrl" || ic == "shadowUrl" ) {
        var s = iconinf[ic];
        iconinf[ic] = String(s);
      }
    }
    return iconinf;
  }

});
